# -*- encoding: utf-8 -*-
'''
Requires http://pypi.python.org/pypi/icalendar/
easy_install icalendar
'''

import urllib2

from amara.thirdparty import json

# Top-level import errors cause an infinite loop problem (see trac #6)
# If this third-party package doesn't exist, report the problem but
# keep on going.
try:
    from icalendar import Calendar, Event
except ImportError, err:
    import warnings
    warnings.warn("Cannot import 'icalendar': %s" % (err,))
    Calendar = Event = NotImplementedError

from akara.services import simple_service

SERVICE_ID = 'http://purl.org/akara/services/demo/ical.json'
@simple_service('POST', SERVICE_ID, 'ical.json', 'application/json')
def ical2json(body, ctype):
    '''
    Convert iCalendar info to Exhibit JSON
    (see: http://www.ibm.com/developerworks/web/library/wa-realweb6/ )

    Sample request:
    * curl --request POST --data-binary "@foo.ics" --header "Content-Type: text/calendar" "http://localhost:8880/ical.json"
    '''
    ids = set()
    entries = []
    cal = Calendar.from_string(body)
    #[ c['UID'] for c in  cal.subcomponents if c.name == 'VEVENT' ]
    for count, component in enumerate(cal.walk()):
        #if count > MAXRECORDS: break
        if component.name != 'VEVENT': continue
        entry = {}
        entry['summary'] = unicode(component['SUMMARY'])
        entry['label'] = entry['summary'] + '_' + str(count)
        entry['start'] = component['DTSTART'].dt.isoformat()
        entry['end'] = component['DTEND'].dt.isoformat()
        if "URL" in component:
            entry['url'] = component['URL']
        # These are Outlook specific(?)
        if "DESCRIPTION" in component:
            entry['description'] = unicode(component['DESCRIPTION'])
        if "UID" in component:
            entry['id'] = unicode(component['UID'])
        if "DTSTAMP" in component:
            entry['timestamp'] = component['DTSTAMP'].dt.isoformat()

        entries.append(entry)
    return json.dumps({'items': entries}, indent=4)





"""Define the Akara logger and ways to configure it

Akara's logging system is built around its use as a server and not as
a library. It logs everything to a single log file, specified in a
configuration file.

When Akara starts it doesn't know the logging destination. During this
bootstrap phase, any errors (like errors in reading the configuration
file) are sent to sys.stderr. Once the logging file is known, use
set_logfile(f) to start sending log messages to that file _in_
_addition_ to the messages to stderr.

Why? Because the server might be in debug mode, where the messages are
both displayed to stderr and logged to the log file. If this is not
the case then call remove_logging_to_stderr() which does just what it
says it does.

At any time you can call set_logfile(f) again to log the messages to a
different file.

Note: for a recipe for filtering the log entries, see:

 * http://stackoverflow.com/questions/1383254/logging-streamhandler-and-standard-streams

"""

#Note: this logger seems reliably multiprocess-friendly, but not a bad idea to bookmark:
#"How should I log while using multiprocessing in Python?": http://stackoverflow.com/questions/641420/how-should-i-log-while-using-multiprocessing-in-python/894284#894284

import sys
import logging

from cStringIO import StringIO

__all__ = ("logger", "set_logfile", "remove_logging_to_stderr")

# Create the logger here but mark it as private.
# The other modules can access this as "akara.logger"
_logger = logging.getLogger('akara')

# Make the log messages look like:
# Jul 21 01:39:03 akara[11754]: [error] Traceback (most recent call last):
_default_formatter = logging.Formatter(
    "%(asctime)s %(name)s[%(process)s]: [%(levelname)s] %(message)s",
    "%b %d %H:%M:%S")

# Make special log levels for stdout and stderr.
# Makes the logging messages easier to read.
STDOUT, STDERR = 22, 21
logging.addLevelName(STDERR, "stderr")
logging.addLevelName(STDOUT, "stdout")



# The current output stream for the Akara server.
# Gets initialized in a bit.
_current_handler = None


def set_logfile(f):
    """Direct (or redirect) error logging to a file handle"""
    global _current_handler
    new_handler = logging.FileHandler(f)
    new_handler.setFormatter(_default_formatter)
    _logger.addHandler(new_handler)

    if _current_handler is not None:
        _logger.removeHandler(_current_handler)
    _current_handler = new_handler

## Part of initialization, to log to stderr
# Set the default logger to stderr
def _init_stderr_handler():
    new_handler = logging.StreamHandler(sys.stderr)
    new_handler.setFormatter(_default_formatter)
    _logger.addHandler(new_handler)
    return new_handler

# Save this so I can remove it for later, if requested
_stderr_handler = _init_stderr_handler()

# Then forget about it. It's still registered in the error handler.
_current_handler = None

# At this point there is logging to stderr, and it cannot be clobbered
# by set_logfile.

# Later on it is possible to remove the stderr handler
def remove_logging_to_stderr():
    "Disable logging to stderr. This cannot be re-enabled."
    global _stderr_handler
    if _stderr_handler is not None:
        _logger.removeHandler(_stderr_handler)
        _stderr_handler = None


# This is a simple redirector.
# It fails if none of your prints end with a "\n".
# Don't do that. ;)
class WriteToLogger(object):
    def __init__(self, loglevel):
        self.loglevel = loglevel
        self.chunks = []
    def write(self, s):
        if s.endswith("\n"):
            text = "".join(self.chunks) + s[:-1]
            _logger.log(self.loglevel, text)
        else:
            self.chunks.append(s)


def redirect_stdio():
    sys.stdin = StringIO("")
    sys.stdout = WriteToLogger(STDOUT)
    sys.stderr = WriteToLogger(STDERR)

########  Access logger

# Use the logging mechanism to deal with access logging

# I think this is a bit more cute than I would like.
# Log at the DEBUG level to akara.access.
# That always writes to the _access_logger because of the .setLevel(DEBUG)
# The log event trickles up to the 'akara' logger.
# That only displays in debug mode (most likely with -X)
#  Downside: it displays with the standard Akara log prefix

_access_logger = logging.getLogger("akara.access")
_access_logger.setLevel(logging.DEBUG)
_access_log_formatter = logging.Formatter("%(message)s")

_access_handler = None

def set_access_logfile(f):
    global _access_handler
    new_access_handler = logging.FileHandler(f)
    new_access_handler.setFormatter(_access_log_formatter)
    _access_logger.addHandler(new_access_handler)
    if _access_handler is not None:
        _access_logger.removeHandler(_access_handler)
    _access_handler = new_access_handler



"""Support library for OpenSearch templates

This is an internal module to Akara. The API may change in the future.

"""

import re
import urlparse
import urllib


__all__ = ["make_template", "apply_template"]

# The OpenSearch format is documented at
#   http://www.opensearch.org/Specifications/OpenSearch/1.1/Draft_4
# it references the URI spec at
#   http://www.ietf.org/rfc/rfc3986.txt
# Note a proposed extension to OpenSearch at:
#   http://www.snellspace.com/wp/?p=369


###############

# I started by looking for existing OpenSearch implementations but
# they didn't work correctly. For example, one tool uses
# urllib.urlsplit to parse the fields, then does template substitution
# of each field. That works only because the library didn't support
# optional "?" fields. That is, consider:

#  >>> urllib.urlsplit("http://example.com/{spam?}/eggs")
#  SplitResult(scheme='http', netloc='example.com',
#               path='/{spam', query='}/eggs', fragment='')

# I started implementing a parser from the spec then realized the spec
# grammar isn't correctly defined. For example, it has:

#  thost = *( host / tparameter )
#   where
#  host  = IP-literal / IPv4address / reg-name

# which means it allows 127.0.0.1127.0.0.1 as an address. While the
# grammar isn't that bad, it's easier to use an approach more like
# what Python's urllib.urlsplit does.

# I also noticed the grammar doesn't allow http://{userid}.myopenid.com/
# as a template, which seemed important for some cases.

# I can't treat this as a simple template grammar and just search for
# the {...} tokens because of Unicode issues. The encoding is field
# specific. The hostname uses IDNA while most of the rest of the
# encoding uses URL-encoded UTF-8.

# The algorithm breaks the template up into parts. Each part is either
# a string (should be a byte string since URLs are not Unicode) or a
# function corresponding to a template lookup. The function will get a
# dictionary of input parameters and it must returns the correctly
# encoded value.

# Template substitution is a merger of either the byte string or the
# result of calling the function with the input parameters.

### Syntax definitions from the relevant specs

# tparameter     = "{" tqname [ tmodifier ] "}"
# tqname = [ tprefix ":" ] tlname
# tprefix = *pchar
# tlname = *pchar

# pchar = unreserved / pct-encoded / sub-delims / ":" / "@"

# unreserved    = ALPHA / DIGIT / "-" / "." / "_" / "~"
# pct-encoded   = "%" HEXDIG HEXDIG
# sub-delims    = "!" / "$" / "&" / "'" / "(" / ")"
#                 / "*" / "+" / "," / ";" / "="

# tmodifier      = "?"

pchar = r"""([A-Za-z0-9._~!$&''()*+,;=""-]|(%[0-9A-Fa-f]{2}))*"""
scheme = r"[a-zA-Z0-9+.-]+"
tparameter = r"""
(?: {{
       ((?P<tprefix> {pchar} ) : )?
       (?P<tlname> {pchar} )
       (?P<tmodifier>\?)?
    }} )
""".format(pchar=pchar)


scheme_pat = re.compile(r"""( {scheme} | {tparameter}):""".format(
    scheme=scheme, tparameter=tparameter), re.X)

tparameter_pat = re.compile(r"""
{{((?P<tprefix> {pchar} ) : )? (?P<tlname> {pchar} ) (?P<tmodifier>\?)? }}
""".format(pchar=pchar), re.X)

# Match either a tparameter or things which aren't in a template
template_pat = re.compile(r"({tparameter}|[^{{]+)".format(tparameter=tparameter), re.X)

# Test if the re match group contains the '?' tmodifier field
def _is_optional(m):
    return m.group("tmodifier") == "?"

##################


# Parse the URI scheme field.
# Looks something like "http:", "ftp:", or "{scheme}:"
# This parses up to the ':' and returns the offset to the ':' and a
# list of template parts.
def _parse_scheme(uri):
    m = scheme_pat.match(uri)
    if m is None:
        i = uri.find(":")
        if i >= 0:
            msg = "URI scheme must be either text or a single template field: %r"
        else:
            msg = "Missing or unparsable URI scheme: %r"
            
        raise TypeError(msg % (uri,))

    if m.group("tlname") is None:
        # Just text. This must be an ASCII byte string.
        return m.end(), [m.group(0).encode("ascii")]

    if _is_optional(m):
        raise TypeError("URI scheme cannot be an optional template variable")
    def convert_scheme(params, tlname=m.group("tlname")):
        # I could make this a more rigorous test for the legal scheme characters
        return params[tlname].encode("ascii")  # the scheme can only be ASCII

    return m.end(), [convert_scheme, ":"]

# Find the end of the network location field. The start is just after the '//'.
# To make things easier, this must be a string with all template {names} removed!
# That's the "xuri", which uses "X"s to replace the template names.
def _find_netloc(xuri, start):
    end = len(xuri)
    # This is the same test in urllib.urlsplit
    for c in "/?#":
        offset = xuri.find(c, start)
        if offset >= 0 and offset < end:
            end = offset
    return end

def _parse_netloc(netloc, xnetloc):
    # Check to see if there's a username/password field.
    # These happen before the '@', if one exists.
    i = xnetloc.find("@")
    if i >= 0:
        # Username/password fields use the normal utf-8 encoding
        # so handle that with the normal template parser.
        for part in _parse_template(netloc[:i]):
            yield part
        yield "@"
        hostname = netloc[i+1:]
        xhostname = xnetloc[i+1:]
    else:
        hostname = netloc
        xhostname = xnetloc

    # There could be a port after the hostname. 
    # Starts with ":", as in  http://localhost:8080/path/
    i = xhostname.find(":")
    if i >= 0:
        port = hostname[i+1:]
        hostname = hostname[:i]
    else:
        # No port specified
        port = None

    if not hostname:
        raise TypeError("Akara requires a hostname in the template")

    # This is tricky. I have to join all of the subfields before doing
    # the idna encoding. This allows u"{hostname}.Espa\u00F1a.com"
    # since the u".Espa\u00F1a.com" does not encode on its own.

    # Create a list of subparts, either:
    #    - strings which are *not* encoded
    #    - a function to look up the value in the dictionary
    subparts = []
    for m in template_pat.finditer(hostname):
        tlname = m.group("tlname")
        if tlname is None:
            subparts.append(m.group(0))
        else:
            if m.group("tmodifier") == "?":
                raise TypeError("URI hostname cannot contain an optional template variable")
            subparts.append(lambda d, tlname=tlname: d[tlname])

    # In the common case this is a string. No need for the extra overhead.
    if len(subparts) == 1 and isinstance(subparts[0], basestring):
        yield subparts[0].encode("idna")

    else:
        # Function to convert, join, and encode based the parts
        def convert_hostname(params, parts=subparts):
            results = []
            for part in parts:
                if isinstance(part, basestring):
                    results.append(part)
                else:
                    results.append(part(params))
            result = "".join(results)
            return result.encode("idna")
        yield convert_hostname

    # And finally, the port.
    if port is None:
        return

    # If it's just a number, return the number (and the ":" I had removed)
    if port.isdigit():
        yield ":" + port
        return

    # Otherwise it's a parameter. Make sure it's only a paramter
    m = tparameter_pat.match(port)
    if m is None:
        raise TypeError("Port must be either a number or a template name")
    if m.end() != len(port):
        raise TypeError("Port may not contain anything after the template name")
    tlname = m.group("tlname")
    if _is_optional(m):
        extract = lambda params, tlname=tlname: params.get(tlname, "")
    else:
        extract = lambda params, tlname=tlname: params[tlname]
        
    def convert_port(params, extract=extract, tlname=tlname):
        value = extract(params)
        if isinstance(value, int):
            # Allow people to pass in a port number as an integer
            return ":%d" % (value,)
        if value == "":
            # No port given? Use the default. (Don't include the ':' here.)
            return ""
        if value.isdigit():
            return ":" + value
        raise TypeError("Port template parameter %r is not an integer (%r)" %
                        (tlname, value))
    yield convert_port

# Handle the text fields which are escaped via URL-encoded UTF-8
def _parse_template(template):
    for m in template_pat.finditer(template):
        if m.group("tlname") is None:
            # "ascii" to ensure that no Unicode characters are in the template
            yield m.group(0).encode("ascii") # You must pre-encode non-ASCII text yourself
        else:
            if _is_optional(m):
                def convert_scheme(params, tlname=m.group("tlname")):
                    return urllib.quote_plus(params.get(tlname, "").encode("utf8"))
            else:
                def convert_scheme(params, tlname=m.group("tlname")):
                    return urllib.quote_plus(params[tlname].encode("utf8"))
            yield convert_scheme


def decompose_template(uri):
    """Internal function to break down an OpenSearch template into its Template terms"""
    # For use in Akara, the scheme and host name are required, and the
    # "uri" syntax is defined from RFC 3986 + OpenSearch templates.

    # I'll make life easier by working with a string without the {} templates.
    def check_and_replace_template_field(m):
        if m.group("tprefix") is not None:
            raise TypeError("Template prefix not supported in Akara (in %r)" % (m.group(0),))
        
        if m.group("tlname") == "":
            raise TypeError("Empty template variable in %r" % (uri,))
        return "X" * (m.end() - m.start())
            
    xuri = tparameter_pat.sub(check_and_replace_template_field, uri)

    # Make sure that no "{" or "}" characters are present!
    if "{" in xuri:
        raise TypeError(
            "Unexpected '{' found in URI at position %d)" % (xuri.index("{")+1,))
    if "}" in xuri:
        raise TypeError(
            "Unexpected '}' found in URI at position %d" % (xuri.index("}")+1,))

    parts = []

    # "http:", "ftp:", "{scheme}:"
    start, subparts = _parse_scheme(uri)
    parts.extend(subparts)

    # Check for the "//" in things like "http://example.com"
    if uri[start:start+2] != "//":
        raise TypeError("Missing required '//' in URI (scheme and hostname must be given)")
    assert isinstance(parts[-1], basestring)
    # This is either something like ["http:"] or something like [<function>. ":"]
    # Optimize by merging the last item with the "//"
    parts[-1] += "//"
    start += 2

    # [tuserinfo "@"] thost [ ":" tport ]
    # The OpenSearch template makes this harder to find. I have to consider:
    #    http://example.com?spam
    #    http://{host?}?spam
    #    http://example.com/spam
    #    http://example.com#spam
    #    ftp://userid:password@ftp
    #    ftp://{userid?}:{password?}@{userid}.example.com/

    # Since I've replaced the templates with "X"s, this becomes a lot easier.
    end = _find_netloc(xuri, start)
    netloc  =  uri[start:end]
    xnetloc = xuri[start:end]

    # The userid portion is encoded different than the hostname
    parts.extend(_parse_netloc(netloc, xnetloc))

    # And the rest is a simple encoding
    parts.extend(_parse_template(uri[end:]))

    return parts


class Template(object):
    """A parsed OpenSearch Template object.

    Use 'make_template(template_string)' to make a Template instance.
    """
    def __init__(self, template, terms):
        """You should not call this constructor directly."""
        self.template = template
        self.terms = terms
    def substitute(self, **kwargs):
        """Use kwargs to fill in the template fields.

        Keywords unknown to the template ignored.
        """
        #XXX: this used to use kwargs, but that's not a good idea because not all 
        results = []
        for term in self.terms:
            if isinstance(term, basestring):
                results.append(term)
            else:
                results.append(term(kwargs))
        return "".join(results)

def make_template(template):
    """Given an OpenSearch template, return a Template instance for it.

    >>> template = make_template('http://localhost/search?q={term}')
    >>> template.substitute(term='opensearch syntax')
    'http://localhost/search?q=opensearch+syntax'
    >>>
    """
    terms = decompose_template(template)
    return Template(template, terms)

def apply_template(template, **kwargs):
    """Apply the kwargs to the template fields and return the result

    >>> apply_template('http://{userid}.example.com/status', userid='anonymous')
    'http://anonymous.example.com/status'
    >>>

    Note: it's very common for URI components to be invalid as Python IDs.
    In that case you have to invoke as follows:
    
    apply_template(tpl, **{'a-b': 'cd'})
    """
    t = make_template(template)
    return t.substitute(**kwargs)




import os
import tempfile
import httplib
import urllib, urllib2
import base64
from functools import wraps
from string import Template
from wsgiref.util import request_uri
from itertools import islice, dropwhile

#from amara.lib.iri import *
from amara.lib.iri import absolutize, join#, split_uri_ref
from amara.lib.util import first_item

from akara import logger
from akara import global_config

def status_response(code):
    """
    Given an int or string, return the HTTP status line
    """
    # If code is already a full status line, let it pass through,
    # but if just a response code as a string, convert to int
    try:
        c = int(code)
    except ValueError:
        c = code

    if isinstance(c,int):
        ret = '%i %s'%(c, httplib.responses[c])
    else: # string/unicode
        ret = c
    return ret

# Convert a lower case header name into its RFC 2616-defined equivalent.
# Useful when bridging httplib2 and WSGI.
normalize_http_header_name = lambda h: '-'.join([s.capitalize() for s in h.split('-')])

class iterwrapper:
    """
    Wraps the response body iterator from the application to meet WSGI
    requirements.
    """
    def __init__(self, wrapped, responder):
        """
        wrapped - the iterator coming from the application
        response_chunk_handler - a callable for any processing of a
            response body chunk before passing it on to the server.
        """
        self._wrapped = iter(wrapped)
        self._responder = responder(self._wrapped)
        if hasattr(wrapped, 'close'):
            self.close = self._wrapped.close

    def __iter__(self):
        return self

    def next(self):
        return self._responder.next()


def geturl(environ, relative=''):
    """
    Constructs a portable URL for your application.  If relative is omitted or '',
    Just return the current base URL. Otherwise resolve the relative portion against
    the present base and return the resulting URL.

    (Inspired by url functions in CherryPy and Pylons, and Ian Bicking's code in PEP 333)

    If you have a proxy that forwards the HOST but not the original HTTP request path
    you might have to set akara.proxy-base in environ (e.g. through .ini)  See

    http://wiki.xml3k.org/Akara/Configuration
    """

    #Manually set proxy base URI for non-well-behaved proxies, such as Apache < 1.3.33,
    #Or for cases where the proxy is not mounted at the root of a host, and thus the original
    #request path info is lost
    if environ.get('akara.proxy-base'):
        url = environ['akara.proxy-base']
        if relative: url = Uri.Absolutize(relative, url)
        return url

    url = environ['wsgi.url_scheme']+'://'
    #Apache 1.3.33 and later mod_proxy uses X-Forwarded-Host
    if environ.get('HTTP_X_FORWARDED_HOST'):
        url += environ['HTTP_X_FORWARDED_HOST']
    #Lighttpd uses X-Host
    elif environ.get('HTTP_X_HOST'):
        url += environ['HTTP_X_HOST']
    elif environ.get('HTTP_HOST'):
        url += environ['HTTP_HOST']
    else:
        url += environ['SERVER_NAME']

        if environ['wsgi.url_scheme'] == 'https':
            if environ['SERVER_PORT'] != '443':
               url += ':' + environ['SERVER_PORT']
        else:
            if environ['SERVER_PORT'] != '80':
               url += ':' + environ['SERVER_PORT']

    #Can't use the more strict Uri.PercentEncode because it would quote the '/'
    url += urllib.quote(environ.get('SCRIPT_NAME', '').rstrip('/')) + '/'
    if relative: url = Uri.Absolutize(relative, url)
    return url


def guess_self_uri(environ):
    return absolutize(environ['SCRIPT_NAME'].rstrip('/'), request_uri(environ, include_query=False))


def in_akara():
    'Returns True if the current process is running as an Akara module'
    return 'pid_file' in global_config.__dict__


def find_peer_service(peer_id, environ=None):
    '''
    DEPRECATED! Use discover_service() and work with the resulting query_template property
    Find a peer service endpoint, by ID, mounted on this same Akara instance
    
    Must be caled from a running akara service, and it is highly recommended to call
    at the top of service functions, or at least before the request environ has been manipulated
    '''
    if not in_akara():
        raise RuntimeError('find_peer_service is meant to be called from within Akara process space')
    from akara.registry import _current_registry
    from akara import request
    if environ:
        serverbase = guess_self_uri(environ)
    else:
        serverbase = getattr(global_config, 'server_root')
    for (path, s) in _current_registry._registered_services.iteritems():
        if s.ident == peer_id:
            return join(serverbase, '..', path)
    return None


def discover_service(sid):
    '''
    Discover a service in the current environment with the given ID
    It might be a peer, mounted on this same Akara instance, or it might
    Be a proxy for a remote Web service
    
    Must be called from a running akara service

    Note if you were using the deprecated
    find_peer_service(peer_id, environ=None) switch to this:
    >>> s = discover_service(peer_id)
    >>> url = join(server_call_base(environ), '..', path)
    '''
    if not in_akara():
        raise RuntimeError('find_peer_service is meant to be called from within Akara process space')
    from akara.registry import _current_registry
    from akara import request
    for (path, s) in _current_registry._registered_services.iteritems():
        if s.ident == peer_id:
            return s
    return None


def server_call_base(environ=None):
    '''
    Determine the best base URL for calling a peer service on this Akara instance
    
    Must be called from a running akara service, and it is highly recommended to call
    at the top of service functions, or at least before the request environ has been manipulated
    '''
    if not in_akara():
        raise RuntimeError('find_peer_service is meant to be called from within Akara process space')
    if environ:
        serverbase = guess_self_uri(environ)
    else:
        serverbase = getattr(global_config, 'server_root')
    return serverbase


def http_method_handler(method):
    '''
    A decorator maker to flag a function as suitable for a given HTTP method
    '''
    def wrap(f):
        #@wraps(f)
        #def wrapper(*args, **kwargs):
        #    return f()
        f.method = method
        return f
    return wrap


class wsgibase(object):
    def __init__(self):
        self._method_handlers = {}
        if not hasattr(self, 'dispatch'):
            self.dispatch = self.dispatch_by_lookup
        #if not hasattr(self, 'dispatch'):
        #    self.dispatch = self.dispatch_by_lookup if hasattr(self, '_methods') else self.dispatch_simply
        for obj in ( getattr(self, name) for name in dir(self) ):
            method = getattr(obj, 'method', None)
            if method:
                self._method_handlers[method] = obj
        return

    def __call__(self, environ, start_response):
        self.environ = environ
        self.start_response = start_response
        return self

    def __iter__(self):
        func = self.dispatch()
        if func is None:
            response_headers = [('Content-type','text/plain')]
            self.start_response(response(httplib.METHOD_NOT_ALLOWED), response_headers)
            yield 'HTTP method Not Allowed'
        else:
            yield func()

    def dispatch_simply(self):
        func = 'do_%s' % self.environ['REQUEST_METHOD']
        if not hasattr(self, func):
            return None
        else:
            return func

    def dispatch_by_lookup(self):
        return self._method_handlers.get(self.environ['REQUEST_METHOD'])

    def parse_fields(self):
        s = self.environ['wsgi.input'].read(int(self.environ['CONTENT_LENGTH']))
        return cgi.parse_qs(s)


def extract_auth(environ):
    '''
    Extract auth creds (HTTP basic only, for now) from the incoming request and return the
    (username, password)

    environ - The usual WSGI structure. Note: if you are using simple_service,
    in Akara services available as akara.request.environ, or perhaps passed right
    into the handler
    top - top URL to be used for this auth.
    '''
    #Useful: http://www.voidspace.org.uk/python/articles/authentication.shtml
    auth = environ.get('HTTP_AUTHORIZATION')
    if not auth: return None
    scheme, data = auth.split(None, 1)
    if scheme.lower() != 'basic':
        raise RuntimeError('Unsupported HTTP auth scheme: %s'%scheme)
    username, password = data.decode('base64').split(':', 1)
    return username, password


def copy_auth(environ, top, realm=None):
    '''
    Get auth creds (HTTP basic only, for now) from the incoming request and return an
    HTTP auth handler for urllib2.  This handler allows you to "forward" this auth to
    remote services

    environ - The usual WSGI structure. Note: if you are using simple_service,
    in Akara services available as akara.request.environ, or perhaps passed right
    into the handler
    top - top URL to be used for this auth.
    '''
    #Useful: http://www.voidspace.org.uk/python/articles/authentication.shtml
    creds = extract_auth(environ)
    if creds:
        username, password = creds
    else:
        return None
    
    password_mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
    # HTTPPasswordMgr top must omit any URL components before the host (i.e. no scheme and no auth info in the authority section)
    #(scheme, authority, path, query, fragment) = split_uri_ref(top)
    #auth, host, port = split_authority(authority)
    #auth_top_url = (host + ':' + port if port else host) + path
    #print >> sys.stderr, 'Auth creds: %s:%s (%s)'%(username, password, auth_top_url)
    logger.debug('Auth creds: %s:%s (%s)'%(username, password, top))
    
    # Not setting the realm for now, so use None
    #password_mgr.add_password(None, auth_top_url, username, password)
    password_mgr.add_password(None, top, username, password)
    #password_handler = urllib2.HTTPDigestAuthHandler(password_mgr)
    password_handler = urllib2.HTTPBasicAuthHandler(password_mgr)
    return password_handler


def header_credentials(username, password, headers=None):
    '''
    httplib2's simple HTTP auth support is great, but it doesn't recognize every case
    in which auth is needed, sometimes because of compliance issues on the remote site*
    
    Also, there are unusual cases where you want to always send the auth header,
    without first waiting for  401 challenge
    
    This function helps with these issues by unconditionally setting up httplib2 headers
    for Basic authentication
    
    >>> username = 'me@example.com'
    >>> password = 'password'
    >>> H = httplib2.Http()
    >>> auth_headers = header_credentials(username, password)
    >>> response, content = H.request(url, 'GET', headers=auth_headers)
    
    * For an example of such issues: http://pyre.posterous.com/accessing-posterous-api-in-python
    '''
    credentials = "Basic " + base64.b64encode("%s:%s"%(username, password))
    if headers:
        headers.update({ 'Authorization': credentials })
    else:
        headers = { 'Authorization': credentials }
    return headers


CHUNKLEN = 4096
def read_http_body_to_temp(environ, start_response):
    '''
    Handle the reading of a file from an HTTP message body (file pointer from wsgi.input)
    in chunks to a temporary file
    Returns the file path of the resulting temp file
    '''
    clen = int(environ.get('CONTENT_LENGTH', None))
    if not clen:
        raise ContentLengthRequiredError()
    http_body = environ['wsgi.input']
    temp = tempfile.mkstemp(suffix=".dat")
    while clen != 0:
        chunk_len = min(CHUNKLEN, clen)
        data = http_body.read(chunk_len)
        if data:
            #assert chunk_len == os.write(temp[0], data)
            written = os.write(temp[0], data)
            #print >> sys.stderr, "Bytes written to file in this chunk", written
            clen -= len(data)
        else:
            clen = 0
    os.fsync(temp[0]) #is this needed with the close below?
    os.close(temp[0])
    return temp[1]

#Convert WSGI environ headers to a plain header list (e.g. for forwarding request headers)
#Copied from webob.
_key2header = {
    'CONTENT_TYPE': 'Content-Type',
    'CONTENT_LENGTH': 'Content-Length',
    'HTTP_CONTENT_TYPE': 'Content_Type',
    'HTTP_CONTENT_LENGTH': 'Content_Length',
}

#Skipping User-Agent is actually Moin-specific, since Moin seems to react to different UAs, and e.g. gives 403 errors in response to Curl's UA
_skip_headers = [
    'HTTP_HOST',
    'HTTP_ACCEPT',
    'HTTP_USER_AGENT',
]

def _trans_key(key, exclude=[]):
    if not isinstance(key, basestring):
        return None
    elif key in _key2header:
        #Do NOT copy these special headers (change from Webob)
        return None
        #return _key2header[key]
    elif key in _skip_headers or key in exclude:
        return None
    elif key.startswith('HTTP_'):
        return key[5:].replace('_', '-').title()
    else:
        return None


def copy_headers(environ,exclude=[]):
    header_list = []
    for k, v in environ.iteritems():
        pure_header = _trans_key(k,exclude)
        if pure_header:
            #FIXME: does this account for dupe headers in the inbound WSGI?
            header_list.append((pure_header, v))
    return header_list


def copy_headers_to_dict(environ, exclude=[]):
    headers = {}
    for k, v in environ.iteritems():
        pure_header = _trans_key(k,exclude)
        if pure_header:
            #FIXME: does this account for dupe headers in the inbound WSGI?
            headers[pure_header] = v
    return headers


def requested_imt(environ):
    # Choose a preferred media type from the Accept header, using application/json as presumed
    # default, and stripping out any wildcard types and type parameters
    #
    # FIXME Ideally, this should use the q values and pick the best media type, rather than
    # just picking the first non-wildcard type.  Perhaps: http://code.google.com/p/mimeparse/
    accepted_imts = []
    accept_header = environ.get('HTTP_ACCEPT')
    if accept_header :
        accepted_imts = [ mt.split(';')[0] for mt in accept_header.split(',') ]
    accepted_imts.append('application/json')
    #if logger: logger.debug('accepted_imts: ' + repr(accepted_imts))
    imt = first_item(dropwhile(lambda x: '*' in x, accepted_imts))
    return imt

#
# ======================================================================
#                       Exceptions
# ======================================================================

# Base exception used to indicate errors.  Rather than replicating tons
# of error handling code, these errors are raised instead.  A top-level
# exception handler catches them and then generates some kind of 
# appropriate HTTP response.  Positional arguments (if any)
# are just passed to the Exception base as before.  Keyword arguments
# are saved in a local dictionary.  They will be used to pass parameters
# to the Template strings used when generating error messages.

class HttpError(Exception): 
    def __init__(self,*args,**kwargs):
        Exception.__init__(self,*args)
        self.parms = kwargs

class BadTargetError(HttpError): pass
class HTTPAuthorizationError(HttpError): pass
class MoinAuthorizationError(HttpError): pass
class UnexpectedResponseError(HttpError): pass
class MoinMustAuthenticateError(HttpError): pass
class MoinNotFoundError(HttpError): pass
class ContentLengthRequiredError(HttpError): pass
class GenericClientError(HttpError): pass
class ConflictError(HttpError): pass



"""Parse the Akara config file (in .ini format) and get system settings from it

This is an internal module and should not be used by other libraries.

"""
import os
import logging
import inspect

class Error(Exception):
    pass

DEFAULT_SERVER_CONFIG_FILE = os.path.expanduser("~/.config/akara.conf")

class AkaraDefault:
    Listen = 8880
    ConfigRoot = "~/.local/lib/akara"
    #"ServerRoot": None
    #"InternalServerRoot": None
    PidFile = "logs/akara.pid"

    MinSpareServers = 5
    MaxSpareServers = 10
    MaxServers = 150
    MaxRequestsPerServer = 10000

    ModuleDir = 'modules'
    ModuleCache = 'caches'
    ErrorLog = 'logs/error.log'
    AccessLog = 'logs/access.log'
    LogLevel = 'INFO'



_valid_log_levels = {
    "CRITICAL": logging.CRITICAL,
    "ERROR": logging.ERROR,
    "WARN": logging.WARN,
    "WARNING": logging.WARNING,
    "INFO": logging.INFO,
    "DEBUG": logging.DEBUG,
    }


log = logging.getLogger("akara.server")

def _add_akara_defaults(akara_namespace):
    # in-place modify the namespace
    for name, value in inspect.getmembers(AkaraDefault):
        if name.startswith("_"):
            continue
        if not hasattr(akara_namespace, name):
            setattr(akara_namespace, name, value)
    
def read_config(config_file=None):
    "Read an Akara configuration file and return the parsed settings"
    if config_file is None:
        config_file = DEFAULT_SERVER_CONFIG_FILE
    try:
        config = open(config_file).read()
    except IOError, err:
        raise Error("""\
Could not open Akara configuration file:
   %s
To set up the default configuration file and directories use "akara setup"\
""" % (err,))

    # XX better error reporting
    try:
        code = compile(config, config_file, "exec")
    except SyntaxError, err:
        raise Error("""\
Could not parse Akara configuration file:
   %s
because: %s""" % (config_file, err))
         
    namespaces = dict(__builtins__ = None, __name__ = "akara.conf")
    exec code in namespaces

    errmsg = None
    if "Akara" not in namespaces:
        raise Error("Configuration file missing required 'Akara' definition")

    _add_akara_defaults(namespaces["Akara"])
    settings = _extract_settings(namespaces["Akara"])
    return settings, namespaces

def _extract_settings(config):
    """Get parsed setting information from a config object

    This does sanity checking on the input (like that port numbers
    must be positive integers) and converts things into the
    appropriate data type (like integers).
    """
    # Some helper functions to get typed fields with good error reporting
    def get(key):
        value = getattr(config, key, None)
        if value is None:
            raise Error("Required 'Akara' configuration %r is missing" % (key,))
        return value
    
    def getstring(key):
        value = get(key)
        if not isinstance(value, basestring):
            raise Error("'Akara' configuration %r must be a string, not %r" %
                        (key, value))
        return value

    def getint(key):
        value = get(key)
        try:
            return int(value)
        except ValueError:
            raise Error("'Akara' configuration %r must be an integer, not %r" % 
                        (key, value))
        
    def getpositive(key):
        value = get(key)
        if value <= 0:
            raise Error(
                "'Akara' configuration %r must be a positive integer, not %r" %
                (key, value))
        return value

    def getnonnegative(key):
        value = getint(key)
        if value <= 0:
            raise Error(
                "'Akara' configuration %r must be a non-negative integer, not %r" %
                (key, value))
        return value


    settings = {}

    # The value for 'Listen' can be:
    #    <port> as in 8080
    # -or-
    #    <host>:<port> as in "localhost:8081"
    addr = get('Listen')
    if isinstance(addr, int):
        host, port = ("", addr)
    else:
        if ':' in addr:
            host, port_s = addr.rsplit(':', 1)
        else:
            host, port_s = '', addr
        try:
            port = int(port_s)
            if port <= 0:
                raise ValueError
        except ValueError:
            raise Error("Listen port must be a positive integer, not %r" % port_s)

    settings["server_address"] = (host, port)

    # Used to contract the full OpenSearch template to a given service.
    # If not present, use the Listen host and port.
    #  (And if the host isn't present, use 'localhost'. It's not a good
    #  default but I'm not going to do a FQDN lookup here since that has
    #  side effects. Basically, if you need the name right, then set it.)
    try:
        server_root = getstring('ServerRoot')
    except Error:
        if port == 80:
            fmt = "http://%(host)s/"
        else:
            fmt = "http://%(host)s:%(port)s/"
        server_root = fmt % dict(host = (host or "localhost"), port = port)
        
    # Uses only when an Akara service wants to call another Akara service.
    # Needed for the (rare) cases when the listen server has a different
    # local name than the published server.
    try:
        internal_server_root = getstring('InternalServerRoot')
    except Error:
        internal_server_root = server_root
        
    settings["server_root"] = server_root
    settings["internal_server_root"] = internal_server_root

    config_root = getstring('ConfigRoot')
    config_root = os.path.expanduser(config_root)
    settings["config_root"] = os.path.abspath(config_root)

    pid_file = getstring('PidFile')
    settings["pid_file"] = os.path.join(config_root, pid_file)

    error_log = getstring('ErrorLog')
    settings["error_log"] = os.path.join(config_root, error_log)

    access_log = getstring('AccessLog')
    settings["access_log"] = os.path.join(config_root, access_log)

    module_dir = getstring("ModuleDir")
    settings["module_dir"] = os.path.join(config_root, module_dir)
    
    module_cache = getstring("ModuleCache")
    settings["module_cache"] = os.path.join(config_root, module_cache)

    log_level_orig = getstring('LogLevel')
    log_level_s = log_level_orig.upper()
    if log_level_s in _valid_log_levels:
        log_level = _valid_log_levels[log_level_s]
    else:
        raise Error(
            "global setting 'LogLevel' is %r but must be one of: %s" %
            (log_level_s, ", ".join(map(repr, _valid_log_levels))))
                    
    settings["log_level"] = log_level



    settings["max_servers"] = getpositive("MaxServers")
    settings["min_spare_servers"] = getnonnegative("MinSpareServers")
    settings["max_spare_servers"] = getnonnegative("MaxSpareServers")
    if settings["max_spare_servers"] < settings["min_spare_servers"]:
        raise Error("MaxSpareServers (%r) must be greater than MinSpareServers (%r)" %
                    (settings["max_spare_servers"], settings["min_spare_servers"]))
    settings["max_requests_per_server"] = getpositive("MaxRequestsPerServer")

    return settings



"""Provide caching of requests to Akara services.

The library implements a cache object that is wrapped around
an existing Akara service.   For example, suppose there is
an Akara service instance called http://myservices.com/bookprice
that returns the prices of books. A cache can be created as 
follows:

    from akara.caching import cache
    PRICE_CACHE = cache("http://myservices.com/bookprice")

The resulting PRICE_CACHE object then provides its own
API for issuing requests.  To issue a GET request, you would
do something like this:

    price = PRICE_CACHE.get(title="Python Essential Reference")

When this operation is performed, it is translated into a request
to the associated Akara mount point for the indicated service.
For example, if the "http://myservices.com/bookprice" service
has an Akara mountpoint of http://myhost/ak/bookprice.xml, then
a GET request for the following URL is issued:

    http://myhost/ak/bookprice.xml?title=Python%20Essential%20Reference

The actual mountpoint used is transparent.  Users specify a cache
according to a high-level service ID. Akara will figure out the
mount point internally using its registry or some other means.

Initial implementation:

The object returned by PRICE_CACHE.get() emulates that returned
by urllib2.urlopen().   That is, it is a file-like object that
can be read to obtain content.  The .info() method returns
information about returned HTTP headers.   If any errors occur
(e.g., 404), it results in an exception just like urllib2.urlopen().

The cache does not record HTTP errors.  Only the results of 
successful requests (200 OK) are stored.
"""

import urllib, urllib2
import os
import shutil
import sys
import base64
import hashlib
import cPickle as pickle
import time

from akara import registry
from akara import global_config

# File object returned to clients on cache hit.  A real file, but with an info() method
# to mimic that operation on file-like objects returned by urlopen().

class CacheFile(file):
    def __init__(self,*args, **kwargs):
        file.__init__(self,*args,**kwargs)
        self.headers = None
        self.code = 200
        self.url = None
        self.msg = "OK"
    def getcode(self):
        return self.code
    def geturl(self):
        return self.url
    def info(self):
        return self.headers

# Utility function to remove the oldest entry from a cache directory.
# Look at the modification dates for metadata files
def remove_oldest(cachedir):
    files = (name for name in os.listdir(cachedir) if name.endswith(".p"))
    paths = (os.path.join(cachedir,name) for name in files)
    time_and_paths = ((os.path.getmtime(path),path) for path in paths)
    oldest_time, path = min(time_and_paths)
    
    # Remove the oldest entry
    os.remove(path)
    
class cache(object):
    def __init__(self,ident,maxentries=65536,expires=15*60,opener=None):
        """Create a cache for another Akara service.

           ident is the Akara service ID
           maxentries is the maximum number of cache entries (approximate)
           expires is the time in seconds after which entries expire
           opener is an alternative URL opener.  By default urllib2.urlopen is used.
        """

        self.ident = ident
        if opener is None:
            opener = urllib2.urlopen
        self.opener = opener
        self.maxentries = maxentries
        self.expires = expires
        self.maxperdirectory = maxentries / 256
        self.serv = None
        self.initialized = False

    # Internal method that locates the Akara service description and sets up a
    # base-URL for making requests.   This can not be done in __init__() since
    # not all Akara services have been initialized at the same __init__() executes.
    # Instead, it has to be done lazily after all services are up and running.
    # In this implementation, the _find_service() method gets invoked upon the
    # first use of the cache
 
    def _find_service(self):
        self.serv = registry.get_a_service_by_id(self.ident)
        if not self.serv:
            raise KeyError("Nothing known about service %s" % ident)
#        print >>sys.stderr,"CACHE: %s at %s\n" % (self.ident, self.serv.path)

        hostname,port = global_config.server_address
        if not hostname:
            hostname = "localhost"
        self.baseurl = "http://%s:%d/%s" % (hostname, port, self.serv.path)
#        print >>sys.stderr,"CACHE: %s\n" % self.baseurl

    # Method that makes the cache directory if it doesn't yet exist
    def _make_cache(self):

        # Make sure the cache directory exists
        if not os.path.exists(global_config.module_cache):
            try:
                os.mkdir(global_config.module_cache)
            except OSError:
                pass    # Might be a race condition in creating.  Ignore errors, but follow up with an assert
            assert os.path.exists(global_config.module_cache),"%s directory can't be created" % (global_config.module_cache)

        self.cachedir = os.path.join(global_config.module_cache,self.serv.path)
        if not os.path.exists(self.cachedir):
            try:
                os.mkdir(self.cachedir)
            except OSError:
                # This exception handler is here to avoid a possible race-condition in startup.
                # Multiple server instances might enter here at the same time and try to create directory
                pass
            assert os.path.exists(self.cachedir), "Failed to make module cache directory %s" % self.cachedir

    # Method that initializes the cache if needed
    def _init_cache(self):
        self._find_service()
        self._make_cache()
        self.initialized = True

    def get(self,**kwargs):
        """Make a cached GET request to an Akara service.  If a result can be
           found in the cache, it is returned.  Otherwise, a GET request is issued
           to the service using urllib2.urlopen().  The result of this operation 
           mimics the file-like object returned by urlopen().   Reading from the file
           will return raw data.  Invoking the .info() method will return HTTP
           metadata about the request."""

        # Check to see if the cache has been initialized or not.  This has to be done here since
        # it is currently not possible to fully initialize the cache in __init__().
        if not self.initialized:
            self._init_cache()

        # This is a sanity check.  If the cache is gone, might have to rebuild it
        if not os.path.exists(self.cachedir):
            self._make_cache()

        #  Make a canonical query string from the arguments (guaranteed
        #  to be the same even if the keyword argumenst are specified in
        #  in an arbitrary order)
        #
        
        query = "&".join(name+"="+urllib.quote(str(value)) for name,value in sorted(kwargs.items()))

        # Take the query string and make a SHA hash key pair out of it.  The general idea here
        # is to come up with an identifier that has a reasonable number of bits, but which is extremely
        # unlikely to collide with other identifiers.  It would be extremely unlikely that the query
        # would have a collision with other entries
        
        shadigest = hashlib.sha1()
        shadigest.update(query)

        # Create a unique cache identifier from the digest.

        identifier = shadigest.hexdigest()

        # Caching operation.  The identifier is split into 2 parts.  The first byte is turned
        # into two hex digits which specify a cache directory.  The remaining bytes are turned
        # into a 47-character filename.
        # specify a cache directory.  The remaining digits specify a filename.  We do this
        # to avoid putting too many files into one big directory (which slows down filesystem operations

        subdir = identifier[:2]
        filename = identifier[2:]

        # Check for the existence of a cache subdirectory.  Make it if neeeded
        cache_subdir = os.path.join(self.cachedir,subdir)
        if not os.path.exists(cache_subdir):
            # Make the cache subdirectory if it doesn't exist
            try:
                os.mkdir(cache_subdir)
            except OSError:
                pass    # Here for possible race condition
            assert os.path.exists(cache_subdir), "Failed to make directory %s" % cache_subdir
            
        # Check for existence of cache file
        cache_file= os.path.join(cache_subdir,filename+".p")
        if os.path.exists(cache_file):
            # A cache hit. Load the metadata file to get the cache information and return it.
            f = CacheFile(cache_file,"rb")
            metaquery,timestamp,url,headers = pickle.load(f)

            # Check to make sure the query string exactly matches the meta data
            # and that the cache data is not too old
            if metaquery == query and (timestamp + self.expires > time.time()):
                # A cache hit and the query matches.  Just return the file we opened.
                # the file pointer should be set to imemdiately after the pickled
                # metadata at the front
                f.headers = headers
                f.url = url
                return f

            # There was a cache hit, but the cache metadata is for a different query (a collision)
            # or the timestamp is out of date.   We're going to remove the cache file and 
            # proceed as if there was a cache miss
            try:
                os.remove(cache_file)
            except OSError:
                pass   # Ignore.  If the files don't exist, who cares?
            
        # Cache miss
        # On a miss, a GET request is issued using the cache opener object
        # (by default, urllib2.urlopen).  Any HTTP exceptions are left unhandled
        # for clients to deal with if they want (HTTP errors are not cached)

        # Before adding a new cache entry. Check the number of entries in the cache subdirectory.
        # If there are too many entries, remove the oldest entry to make room.
        if len(os.listdir(cache_subdir)) >= self.maxperdirectory:
            remove_oldest(cache_subdir)

        # Make an akara request
        url = self.baseurl + "?" + query
        u = self.opener(url)
        
        # If successful, we'll make it here.  Read data from u and store in the cache
        # This is done by initially creating a file with a different filename, fully
        # populating it, and then renaming it to the correct cache file when done.
        cache_tempfile = cache_file + ".%d" % os.getpid()
        f = open(cache_tempfile,"wb")
        pickle.dump((query,time.time(),url,u.info()),f,-1)

        # Write content into the file
        while True:
            chunk = u.read(65536)
            if not chunk: break
            f.write(chunk)
        f.close()

        # Rename the file, open, and return
        shutil.move(cache_tempfile, cache_file)

        # Return a file-like object back to the client
        f = CacheFile(cache_file,"rb")
        metaquery,timestamp,f.url,f.headers = pickle.load(f)
        return f


#
# Method that makes the cache directory if it doesn't yet exist
def make_named_cache(name):
    #serv = registry.get_a_service_by_id(ident)
    #if not serv:
    #    raise KeyError("Nothing known about service %s" % ident)
    # Make sure the cache directory exists
    if not os.path.exists(global_config.module_cache):
        try:
            os.mkdir(global_config.module_cache)
        except OSError:
            pass    # Might be a race condition in creating.  Ignore errors, but follow up with an assert
        assert os.path.exists(global_config.module_cache),"%s directory can't be created" % (global_config.module_cache)

    cachedir = os.path.join(global_config.module_cache, name)
    if not os.path.exists(cachedir):
        try:
            os.mkdir(cachedir)
        except OSError:
            # This exception handler is here to avoid a possible race-condition in startup.
            # Multiple server instances might enter here at the same time and try to create directory
            pass
        assert os.path.exists(cachedir), "Failed to make module cache directory %s" % cachedir
    return cachedir




# -*- encoding: utf-8 -*-
'''
Useful tools for processing markup
'''

import amara
from amara.bindery import html as htmldoc
from amara.lib.util import trim_word_count

from akara.services import simple_service

SERVICE_ID = 'http://purl.org/akara/services/demo/tidy'
@simple_service('POST', SERVICE_ID, 'tidy.xml', 'application/xml',
                writer="xml")
def tidy(body, ctype):
    '''
    Tidy arbitrary HTML (using html5lib)

    Sample request:
    curl --request POST --data-binary "<a>one two <b>three four </b><c>five <d>six seven</d> eight</c> nine</a>" --header "Content-Type: application/xml" "http://localhost:8880/akara.tidy"
    '''
    doc = htmldoc.parse(body)
    return doc


SERVICE_ID = 'http://purl.org/akara/services/demo/trim-word-count'
@simple_service('POST', SERVICE_ID, 'akara.twc.xml', 'application/xml',
                writer="xml-indent")
def akara_twc(body, ctype, max=None, html='no'):
    '''
    Take some POSTed markup and return a version with words trimmed, but intelligently,
    with understanding of markup, so that tags are not counted, and the structure of
    sub-elements included in the same set is preserved.

    max (query parameter) - which is the maximum word count of the resulting text
    html (query parameter) - if 'yes', try to parse the input as HTML

    Sample request:
    curl --request POST --data-binary "<a>one two <b>three four </b><c>five <d>six seven</d> eight</c> nine</a>" --header "Content-Type: application/xml" "http://localhost:8880/akara.twc?max=7"
    '''
    #Raises ValueError
    #Is there a monadic approach we can provide for Akara for error handling?  This cries out for "Maybe"
    #(OK OK, the idea of Maybe, but more of the simple expressiveness of assert)
    max_ = int(max) if max else 500
    if html == 'yes':
        doc = htmldoc.parse(body)
    else:
        doc = amara.parse(body)
    return trim_word_count(doc, max_)




"""Support module to start a local Akara test server"""

import atexit
import os
from os.path import abspath, dirname
import shutil
import signal
import socket
import subprocess
import sys
import tempfile
import urllib2
import urlparse
import httplib

# XXX I only use one function from here. Bring it into this file?
import python_support

######

# Set 'False' to keep the temporary directory used for the server tests
#DELETE_TEMPORARY_SERVER_DIRECTORY = False
DELETE_TEMPORARY_SERVER_DIRECTORY = True
if "AKARA_SAVE" in os.environ:
    DELETE_TEMPORARY_SERVER_DIRECTORY = False


######
ATOM_ENTRIES = os.path.join(dirname(abspath(__file__)), "resource", "atom")
assert os.path.exists(ATOM_ENTRIES), "no atom entries directory?"

# All of the tests use a single server instance.
# This is started the first time it's needed.
# It is stopped during test shutdown.

def _override_server_uri():
    server = os.environ.get("AKARA_TEST_SERVER", None)
    if not server:
        return None
    assert "/" not in server
    return "http://" + server + "/"
    
SERVER_URI = _override_server_uri()
config_root = None
config_filename = None
server_pid = None
server_did_not_start = False

# Create a temporary directory structure for Akara.
# Needs a configuration .ini file and the logs subdirectory.
def create_server_dir(port):
    global config_root, config_filename
    
    config_root = tempfile.mkdtemp(prefix="akara_test_")
    config_filename = os.path.join(config_root, "akara_test.config")

    f = open(config_filename, "w")
    f.write("""
class Akara:
  ConfigRoot = %(config_root)r
  ServerRoot = 'http://dalkescientific.com/'
  InternalServerRoot = 'http://localhost:%(port)s/'
  Listen = 'localhost:%(port)s'
  LogLevel = 'DEBUG'
  MinSpareServers = 3
  # These affect test_server.py:test_restart
  MaxServers = 5
  MaxRequestsPerServer = 5

class atomtools:
  entries = %(atom_entries)r
  feed_envelope = '''<feed xmlns="http://www.w3.org/2005/Atom">
<title>Feed me!</title><id>http://example.com/myfeed</id></feed>'''

class static:
  paths = {
      "resource": %(resource_dir)r,
      "static": %(resource_dir)r + '/static',
      }


# Let the XSLT test reach directly into the filesystem
class xslt:
  uri_space = 'file:///'

MODULES = ["akara.demo.akara_tests",
           "akara.demo.akara_tests",
           "akara.demo.atomtools",
           "akara.demo.calaistools",
           "akara.demo.calweb",
           "akara.demo.collection",
           "akara.demo.echo",
           "akara.demo.icaltools",
           "akara.demo.luckygoogle",
           "akara.demo.markuptools",
           "akara.demo.method_dispatcher",
           "akara.demo.moin2atomentries",
           "akara.demo.moincms",
           "akara.demo.moinrest",
           "akara.demo.oaitools",
           "akara.demo.rdfatools",
           "akara.demo.static",
           "akara.demo.statstools",
           "akara.demo.svntools",
           "akara.demo.unicodetools",
           "akara.demo.wwwlogviewer",
           "akara.demo.xslt",
]           

""" % dict(config_root = config_root,
           port = port,
           atom_entries = os.path.join(ATOM_ENTRIES, "*.atom"),
           resource_dir = os.path.join(dirname(abspath(__file__)), "resource"),
           ))
    f.close()

    os.mkdir(os.path.join(config_root, "logs"))

#FIXME: add back config for:
#[collection]
#folder=/tmp/collection

# Remove the temporary server configuration directory,
# if I created it
def remove_server_dir():
    global server_pid, config_root
    if server_pid is not None:
        # I created the server, I kill it
        os.kill(server_pid, signal.SIGTERM)
        server_pid = None

    if config_root is not None:
        # Very useful when doing development and testing.
        # Would like this as a command-line option somehow.
        if DELETE_TEMPORARY_SERVER_DIRECTORY:
            shutil.rmtree(config_root)
        else:
            print "Test server configuration and log files are in", config_root
        config_root = None

atexit.register(remove_server_dir)

# Start a new Akara server in server mode.
def start_server():
    global server_pid, server_did_not_start
    # There's a PID if the spawning worked
    assert server_pid is None
    # Can only tell if the server starts by doing a request
    # If the request failed, don't try to restart.
    if server_did_not_start:
        raise AssertionError("Already tried once to start the server")

    port = python_support.find_unused_port()
    create_server_dir(port)
    args = ['akara', "--config-file", config_filename, "start"]
    try:
        result = subprocess.call(args)
    except:
        print "Failed to start", args
        raise

    # Akara started, but it might have failed during startup.
    # Report errors by reading the error log
    if result != 0:
        f = open(os.path.join(config_root, "logs", "error.log"))
        err_text = f.read()
        raise AssertionError("Could not start %r:\n%s" % (args, err_text))

    # Akara server started in the background. The main
    # process will only exit with a success (0) if the
    # pid file has been created.
    f = open(os.path.join(config_root, "logs", "akara.pid"))
    line = f.readline()
    f.close()

    # Save the pid information now so the server will be shut down
    # if there are any problems.
    temp_server_pid = int(line)

    # Check to see that the server process really exists.
    # (Is this overkill? Is this portable for Windows?)
    os.kill(temp_server_pid, 0)  # Did Akara really start?

    server_did_not_start = True
    check_that_server_is_available(port)
    server_did_not_start = False

    # Looks like it's legit!
    server_pid = temp_server_pid
    return port

# It takes the server a little while to get started.
# In the worst case (trac #6), top-level import failures
# will loop forever, and the server won't hear requests.
def check_that_server_is_available(port):
    old_timeout = socket.getdefaulttimeout()
    try:
        # How long do you want to wait?
        socket.setdefaulttimeout(20.0)
        try:
            urllib2.urlopen("http://localhost:%d/" % port).read()
        except urllib2.URLError, err:
            print "Current error log is:"
            f = open(os.path.join(config_root, "logs", "error.log"))
            err_text = f.read()
            print err_text
            raise
    finally:
        socket.setdefaulttimeout(old_timeout)

# Get the server URI prefix, like "http://localhost:8880/"
def server():
    global SERVER_URI
    if SERVER_URI is None:
        # No server specified and need to start my own
        port = start_server()
        SERVER_URI = "http://localhost:%d/" % port

    return SERVER_URI

def httplib_server():
    url = server()
    # <scheme>://<netloc>/<path>?<query>#<fragment>
    scheme, netloc, path, query, fragment = urlparse.urlsplit(url)
    assert path in ("", "/"), "Cannot handle path in %r" % url
    assert query == "", "Cannot handle query in %r" % url
    assert fragment == "", "Cannot handle fragment in %r" % url
    assert "@" not in netloc, "Cannot handle '@' in %r" % url
    if ":" in netloc:
        host, port = netloc.split(":")
        port = int(port)
    else:
        host = netloc
        port = 80
    conn = httplib.HTTPConnection(host, port, strict=True)
    return conn



from akara.services import method_dispatcher

@method_dispatcher("http://purl.org/spam", "vikings")
def vikings():
    """Sing the Viking song

    GET: [word=string] returns the song for that word
    POST: returns some other text
    """

@vikings.simple_method("GET", "text/plain")
def vikings_get(word):
    yield "%s, %s, %s, %s\n" % (word, word, word, word)
    yield "%s, %s, %s, %s\n" % (word, word, word, word)
    yield "%s-itty %s!\n" % (word, word)

@vikings.method("POST")
def vikings_post(environ, start_response):
    start_response("200 OK", [("Content-Type", "text/plain")])
    return "That was interesting.\n"



import server_support
from server_support import server, httplib_server

import urllib, urllib2
from urllib2 import urlopen

import amara

def GET3(name, args=None, data=None):
    url = server() + name
    if args:
        url += "?" + urllib.urlencode(args)
    f = urlopen(url, data)
    s = f.read()
    return f.code, f.headers, s

def GET(name, args=None, data=None):
    code, headers, body = GET3(name, args, data)
    assert code == 200, code
    return body

def test_environment():
    code, headers, body = GET3("test_environment")
    assert code == 200
    assert headers["Content-Type"] == "text/plain"
    assert body == "Good!"

def test_set_path():
    body = GET("test.new.path")
    assert body == "Goody!"

def test_image_gif():
    code, headers, body = GET3("test_image_gif")
    assert headers["Content-Type"] == "image/gif"
    assert body == ('GIF89a\x01\x00\x01\x00\x90\x00\x00\xff\x00\x00\x00'
                    '\x00\x00,\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02'
                    '\x02\x04\x01\x00;')

def test_dynamic_content_type():
    code, headers, body = GET3("test_dynamic_content_type")
    assert headers["Content-Type"] == "chemical/x-daylight-smiles"
    assert body == "c1ccccc1O"

def test_unicode_utf8():
    code, headers, body = GET3("test_unicode_utf8")
    assert headers["Content-Type"] == "text/plain; charset=utf-8", headers["Content-Type"]
    assert body == u"\xc5sa bor i G\xf6teborg".encode("utf-8")

def test_unicode_latin1():
    code, headers, body = GET3("test_unicode_latin1")
    assert headers["Content-Type"] == "text/plain", headers["Content-Type"]
    assert body == u"\xc5sa bor i G\xf6teborg".encode("latin1")

def test_unicode_utf16():
    code, headers, body = GET3("test_unicode_utf16")
    assert headers["Content-Type"] == "something/strange", headers["Content-Type"]
    assert body == u"\xc5sa bor i G\xf6teborg".encode("utf16")

def test_xml_utf8():
    code, headers, body = GET3("test_xml_utf8")
    assert headers["Content-Type"] == "application/xml", headers["Content-Type"]
    assert 'encoding="utf-8"?>' in body
    assert u"G\xf6teborg".encode("utf-8") in body, repr(body)
    assert " <something" not in body, body # make sure it is NOT indented

def test_xml_latin1():
    code, headers, body = GET3("test_xml_latin1")
    assert headers["Content-Type"] == "application/xml", headers["Content-Type"]
    assert 'encoding="latin1"?>' in body
    assert u"G\xf6teborg".encode("latin1") in body, repr(body)

def test_xml_utf8_indent():
    code, headers, body = GET3("test_xml_utf8_indent")
    assert headers["Content-Type"] == "application/xml", headers["Content-Type"]
    assert 'encoding="utf-8"?>' in body
    assert u"G\xf6teborg".encode("utf8") in body, repr(body)
    assert " <something" in body, body # make sure it IS indented

def test_list_of_strings():
    code, headers, body = GET3("test_list_of_strings")
    assert headers["Content-Type"] == "text/plain", headers["Content-Type"]
    assert body == "This is a test.\n", body

def test_iterator():
    code, headers, body = GET3("test_iterator")
    assert headers["Content-Type"] == "text/plain", headers["Content-Type"]
    assert body == """\
When shall we three meet again?
In thunder, lightning, or in rain?
When the hurlyburly's done,
When the battle's lost and won.
""", body


def test_args1():
    body = GET("test_args", dict(a="Andrew"))
    assert body == "Hi Andrew and 3"

def test_args2():
    body = GET("test_args", dict(a="Andrew", b="Sara Marie"))
    assert body == "Hi Andrew and Sara Marie"

def test_args2_as_list():
    body = GET("test_args", [("a", "Andrew"), ("b", "Sara Marie")])
    assert body == "Hi Andrew and Sara Marie"

def test_args2_as_list_with_duplicates():
    try:
        body = GET("test_args", [("a", "Andrew"), ("a", "Peter"), ("b", "Sara Marie")])
        raise AssertionError("duplicates should not be allowed!")
    except urllib2.HTTPError, err:
        assert err.code == 400
        s = err.fp.read()
        assert "Using the 'a' query parameter multiple times" in s, s

def test_repeated_args1():
    body = GET("test_repeated_args", dict(a="Andrew"))
    assert body == "Hello ['Andrew'] and 3", repr(body)

def test_repeated_args2():
    body = GET("test_repeated_args", dict(a="Andrew", b="Sara Marie"))
    assert body == "Hello ['Andrew'] and ['Sara Marie']"

def test_repeated_args3():
    body = GET("test_repeated_args", [("a", "Andrew"), ("b", "Sara Marie"), ("a", "Peter")])
    assert body == "Hello ['Andrew', 'Peter'] and ['Sara Marie']"

def test_add_headers():
    code, headers, body = GET3("test_add_headers")
    assert headers["Location"] == "http://freemix.it/"
    # urlopen combines the multiple headers into one.
    # This is completely legit according to the spec.
    assert headers["URL"] == "http://www.xml3k.org/, http://dalkescientific.com/", headers["URL"]


# See if we get back what we went.
def test_echo_simple_get():
    body = GET("test_echo_simple_get", (("a", "1"), ("c", "2"), ("b", "3")))
    assert body == """\
'a' -> '1'
'b' -> '3'
'c' -> '2'
""", body

# Now POST against it, which should fail
def test_echo_simple_get_with_post():
    # this should pass
    GET("test_echo_simple_get", None)
    # this should not pass
    try:
        GET("test_echo_simple_get", None, "Something")
        raise AssertionError("POST to test_echo_simple_get must fail!")
    except urllib2.HTTPError, err:
        assert err.code == 405
        assert "405: Method Not Allowed" in str(err)
        # Required by the HTTP spec
        assert err.headers["Allow"] == "GET, HEAD", err.headers

def test_echo_simple_post():
    body = GET("test_echo_simple_post", data="Heja Sverige!\n")
    assert body == """\
Content-Type: 'application/x-www-form-urlencoded'
Length: 14
Body:
'Heja Sverige!\\n'
""", "Got %r" % (body,)

def test_large_echo():
    body = GET("test_echo_simple_post", data=", ".join(map(str, range(100000))))
    assert "Body:\n'0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, " in body, body[:100]
    assert body.endswith(", 99996, 99997, 99998, 99999'\n"), repr(body[-50:])

def test_echo_simple_post_with_GET():
    try:
        GET("test_echo_simple_post")
        raise AssertionError("GET to test_echo_simple_post must fail!")
    except urllib2.HTTPError, err:
        assert err.code == 405
        assert "405: Method Not Allowed" in str(err)
        # Required by the HTTP spec
        # Also checks that HEAD wasn't added to the header by accident.
        assert err.headers["Allow"] == "POST", err.headers

def test_echo_simple_post_negative_content_length():
    url = server() + "test_echo_simple_post"
    req = urllib2.Request(url, data="I was here.", headers={"Content-Length": "-100"})
    try:
        f = urllib2.urlopen(req)
        raise AssertionError("Not supposed to handle negative lengths")
    except urllib2.HTTPError, err:
        assert err.code == 400

# These have to bypass urllib and work with httplib directly
# in order to generate ill-formatted requests.

def test_good_path():
    h = httplib_server()
    h.request("GET", "/")
    r = h.getresponse()
    assert r.status == 200, r.status

def test_bad_path():
    h = httplib_server()
    # the request is missing the leading '/'
    h.request("GET", "missing_slash")
    r = h.getresponse()
    assert r.status == 400, r.status
    
def test_echo_simple_post_missing_content_length():
    # First, make sure I can call it
    h = httplib_server()
    h.request("POST", "/test_echo_simple_post", "Body",
              {"Content-Length": 4, "Content-Type": "text/plain"})
    r = h.getresponse()
    assert r.status == 200, r.status

    # Try again, this time without a Content-Length
    # (The only way to do that with httplib is to use Content-Length of None)
    h = httplib_server()
    h.request("POST", "/test_echo_simple_post", "Body",
              {"Content-Length": None, "Content-Type": "text/plain"})
    r = h.getresponse()
    # 411 is "Length Required"
    assert r.status == 411, r.status
    

def test_service_no_path():
    code, headers, body = GET3("test_service_no_path")
    assert headers["Content-Type"] == "text/plain"
    assert body == "this uses the default path\n"

def test_service_path():
    code, headers, body = GET3("test_service_path")
    assert headers["Content-Type"] == "text/fancy", headers["Content-Type"]
    assert body == "this specified a path\n"

def test_service_with_a_path():
    # Just checking that it wasn't also registered
    try:
        GET("test_service_with_a_path")
    except urllib2.HTTPError, err:
        assert err.code == 404

## Test that a generator response can still access the query headers
def test_echo_post_headers():
    body = GET("test_echo_post_headers", data="make this a POST")
    assert "HOST ->" in body, body


## Tests for '@method_dispatcher'

def test_dispatching_with_no_methods():
    try:
        GET("test_dispatching_with_no_methods")
        raise AssertionError("test_dispatching_with_no_methods has no methods!")
    except urllib2.HTTPError, err:
        assert err.code == 405, err.code
        assert err.headers.get("Allow", "") == "", repr(err.headers.get("Allow", ""))

def test_dispatching_get_but_with__a_different_name():
    try:
        GET3("test_dispatching_but_with_a_different_name")
        raise AssertionError("test_dispatching_but_with_a_different_name should not work")
    except urllib2.HTTPError, err:
        assert err.code == 404

def test_dispatching_get():
    code, headers, body = GET3("test_dispatching_get")
    assert code == 200
    assert headers["Content-Type"] == "text/plain"
    assert body == "Hi, world!"

def test_dispatching_get_with_arg():
    code, headers, body = GET3("test_dispatching_get", [("a", "Sweden")])
    assert code == 200
    assert headers["Content-Type"] == "text/plain"
    assert body == "Hi, Sweden!", repr(body)
    
def test_dispatching_get_with_wrong_arg():
    try:
        GET("test_dispatching_get", [("b", "Sweden")])
    except urllib2.HTTPError, err:
        # XXX should this be a 400 or 500 error?
        assert err.code == 500, err.code

def test_dispatching_get_with_duplicate_arg():
    try:
        GET("test_dispatching_get", [("a", "Sweden"), ("a", "USA")])
    except urllib2.HTTPError, err:
        assert err.code == 400, err.code
        body = err.read()
        assert "'a' query parameter multiple times" in body, repr(body)

def test_multimethod_get():
    code, headers, body = GET3("test_multimethod", [("a", "Andrew")])
    assert headers["Content-Type"] == "text/fancy"
    assert body == "My name is ['Andrew']. I like 'spam'"
    body = GET("test_multimethod", [("a", "Andrew"), ("b", "lists")])
    assert body == "My name is ['Andrew']. I like ['lists']"
    body = GET("test_multimethod", [("a", "Andrew"),
                                    ("b", "lists"), ("b", "of"), ("b", "things")])
    assert body == "My name is ['Andrew']. I like ['lists', 'of', 'things']"

def test_multimethod_get_bad_args():
    try:
        GET("test_multimethod")
    except urllib2.HTTPError, err:
        assert err.code == 500, err.code
    try:
        GET("test_multimethod", [("qwe", "rty")])
    except urllib2.HTTPError, err:
        assert err.code == 500, err.code


def test_multimethod_post():
    code, headers, body = GET3("test_multimethod", data="some data")
    assert code == 200, code
    assert headers["Content-Type"] == "text/schmancy"
    assert body == """\
You sent 9 bytes of 'application/x-www-form-urlencoded' and 'default value'
First few bytes: 'some data'
""", repr(body)

def test_multimethod_post_with_arg():
    code, headers, body = GET3("test_multimethod", [("a", "something")],
                               data="more data")
    assert code == 200, code
    assert headers["Content-Type"] == "text/schmancy"
    assert body == """\
You sent 9 bytes of 'application/x-www-form-urlencoded' and 'something'
First few bytes: 'more data'
"""

def test_multimethod_post_with_bad_arg():
    try:
        GET3("test_multimethod", [("b", "something")], data="more data")
    except urllib2.HTTPError, err:
        assert err.code == 500
    

def test_multimethod_delete():
    h = httplib_server()
    h.request("DELETE", "/test_multimethod", "[U235]",
              {"Content-Length": 6, "Content-Type": "chemical/x-daylight-smiles"})
    r = h.getresponse()
    assert r.status == 202, r.status

    s = r.read()
    assert s == ('<?xml version="1.0" encoding="utf-8"?>\n'
                 '<nothing city="G\xc3\xb6teborg" name="\xc3\x85sa"><something/></nothing>'), repr(s)


def test_multimethod_teapot():
    h = httplib_server()
    h.request("TEAPOT", "/test_multimethod")
    r = h.getresponse()
    assert r.status == 418, r.status
    s = r.read()
    assert s == "short and stout", repr(s)

# Make a bad request type, see if we get the right responses
def test_multimethod_unknown():
    h = httplib_server()
    # This one isn't supported on the server
    h.request("COFFEEPOT", "/test_multimethod")
    r = h.getresponse()
    assert r.status == 405, r.status
    accept = r.getheader("Allow")
    terms = [s.strip() for s in accept.split(",")]
    terms.sort()
    assert terms == ["DELETE", "GET", "HEAD", "POST", "TEAPOT"], terms

def test_head_vs_get():
    h = httplib_server()
    h.request("GET", "/test_add_headers")
    r = h.getresponse()
    assert r.status == 200, r.status
    get_headers = r.getheaders()
    content_length = r.getheader("content-length")
    assert content_length == "33", content_length
    get_body = r.read()
    assert len(get_body) == 33, (get_body, content_length)

    h.request("HEAD", "/test_add_headers")
    r = h.getresponse()
    assert r.status == 200, r.status
    head_headers = r.getheaders()
    head_body = r.read()
    assert head_body == "", head_body

    assert get_headers == head_headers, (get_headers, head_headers)


# Unicode and XML encoding
def test_method_unicode_latin1():
    code, headers, body = GET3("test_dispatch_unicode")
    assert headers["Content-Type"] == "text/plain; charset=latin1", headers["Content-Type"]
    assert body == u"\xc5sa bor i G\xf6teborg".encode("latin1"), repr(body)

def test_method_unicode_utf8():
    code, headers, body = GET3("test_dispatch_unicode", data="")
    assert headers["Content-Type"] == "text/plain; charset=utf-8", headers["Content-Type"]
    assert body == u"\xc5sa bor i G\xf6teborg".encode("utf8"), repr(body)


expected_utf8 = ('<?xml version="1.0" encoding="utf-8"?>\n'
                 '<nothing city="G\xc3\xb6teborg" name="\xc3\x85sa"><something/></nothing>')
def test_method_xml_utf8():
    code, headers, body = GET3("test_dispatch_xml")
    assert headers["Content-Type"] == "application/xml", headers["Content-Type"]
    assert body == expected_utf8, repr(body)

# Without the byte order marker
expected_utf16 = ('<?xml version="1.0" encoding="utf-16"?>\n'
                  '<nothing city="G\xf6teborg" name="\xc5sa">\n  <something/>\n</nothing>')
def test_method_xml_indent_utf16():
    code, headers, body = GET3("test_dispatch_xml", data="")
    assert headers["Content-Type"] == "application/xml", headers["Content-Type"]

    expected = "\x00".join(expected_utf16) + "\x00"
    assert body == ("\xfe\xff" + expected), repr(body)


# Test the templates
def test_templates():
    url = server()
    f = urlopen(url)
    tree = amara.parse(f)
    nodes = tree.xml_select("//service[@ident='urn:akara.test:template-1']")
    template = (tree.xml_select(
        "//service[@ident='urn:akara.test:template-1']/path/@template")[0].xml_value)
    assert template.endswith("/test.template1?name={name}&language={lang?}"), template

    template = (tree.xml_select(
        "//service[@ident='urn:akara.test:template-2']/path/@template")[0].xml_value)
    assert template.endswith(
        "/test.template2?language={language?}&name={name}&os={os?}"), template

    nodes = (tree.xml_select(
        "//service[@ident='urn:akara.test:template-3']/path/@template"))
    assert len(nodes) == 0, nodes
             
    template = (tree.xml_select(
        "//service[@ident='urn:akara.test:template-4']/path/@template")[0].xml_value)
    assert template.endswith("/test.template4"), template

def test_template_expansion():
    body = GET("test_template5")
    lines = body.splitlines()
    expected = ["server_root: http://dalkescientific.com/",
                "internal_server_root: " + server_support.SERVER_URI]
    for base_uri in (server_support.SERVER_URI,
                     "http://dalkescientific.com/"):
        for suffix in (
            "test.template1?name=Matt&language=kd",
            "test.template2?language=C%2B%2B&name=Matt&os=Linux",
            "test.template4",
            "test.template1?name=%C3%85sa&language=",
            "test.template2?language=C%26C%23&name=%C3%85sa&os=G%C3%B6teborg",
            "test.template4",
            "test.template1?name=%C3%85sa&language=",
            ):
            expected.append( base_uri + suffix )

    assert len(lines) == len(expected), (len(lines), len(expected))

    for expected_line, got_line in zip(expected, lines):
        print repr(expected_line), repr(got_line)
        assert expected_line == got_line, (expected_line, got_line)
    # This is to check line length mismatch
    assert expected == lines, (expected, lines)

## Test the additionally registered external services
def test_additional_services():
    body = GET("test_extra_call", dict(service_id="urn:service_reg:1", x="Andrew"))
    expected = ("URL: %stest.template1?name=Andrew&language=FORTRAN\n"
                "Andrew uses FORTRAN on unix") % (server_support.SERVER_URI,)
    assert body == expected, (body, expected)

    # As a side-effect, the previous called akara_tests.py:_delayed_install()
    body = GET("test_extra_call", dict(service_id="urn:akara.test:extra_echo",
                                       bar="baz"))
    expected = ("URL: %stest_echo_simple_get?foo=baz\n"
                "'foo' -> 'baz'\n") % (server_support.SERVER_URI,)
    assert body == expected, (body, expected)



# -*- encoding: utf-8 -*-
'''
'''
import sys, time
import urllib2
from gettext import gettext as _

from amara.thirdparty import json

from amara.lib.util import *
from amara.tools import rdfascrape

from akara.services import simple_service

URL_REQUIRED = _("The 'url' query parameter is mandatory.")

SERVICE_ID = 'http://purl.org/akara/services/demo/rdfa.json'
@simple_service('GET', SERVICE_ID, 'akara.rdfa.json', 'application/json')
def rdfa2json(url=None):
    '''
    url - the page to check for RDFa
    
    Sample request:
    curl "http://localhost:8880/akara.rdfa.json?url=http://zepheira.com"
    '''
    if url is None:
        raise AssertionError(URL_REQUIRED)
    resources = rdfaparse(url)
    return json.dumps({'items': resources}, indent=4)
    

def rdfaparse(content):
    resources = []
    triples = rdfascrape.rdfascrape(content)
    for count, (s, p, o, dt) in enumerate(triples):
        obj = {}
        obj['label'] = '_' + str(count)
        obj['id'] = '_' + str(count)
        pred = p.split('/')[-1].split('#')[-1]
        if pred == u'dc:date' or dt in [u'xsd:date', u'xs:date', u'http://www.w3.org/2001/XMLSchema' + u'date']:
            # feedparer's internal date parser robustly handles different
            # time formats and returns a 9-tuple
            import feedparser
            normalizeddate = feedparser._parse_date(o)
            obj[pred] = time.strftime("%Y-%m-%dT%H:%M:%S", normalizeddate)
            obj[pred + u'localized'] = time.strftime("%a, %d %b %Y %H:%M:%S", normalizeddate)
        else:
            obj[pred] = o
        resources.append(obj)
    return resources




# -*- encoding: utf-8 -*-
'''
Module name: xslt
Defined REST entry points:

http://purl.org/akara/services/demo/xslt (akara.xslt)
http://purl.org/akara/services/demo/xpath (akara.xpath)

Notes on security:

In order to prevent these services from being used for cross-site attacks,
the URI_SPACE config parameter establishes a (space separated) list of base URIs outside of
which it will not travel.  The default is http://hg.akara.info/
'''

import amara
from amara.xslt import transform
from amara.xpath.util import simplify
from amara.bindery import html
from amara.lib import irihelpers, inputsource

import akara
from akara.services import simple_service
from akara import response

XSLT_SERVICE_ID = 'http://purl.org/akara/services/demo/xslt'
XPATH_SERVICE_ID = 'http://purl.org/akara/services/demo/xpath'


DEFAULT_TRANSFORM = akara.module_config().get('default_transform')
URI_SPACE = akara.module_config().get('uri_space', 'http://github.com/zepheira').split()
#print DEFAULT_TRANSFORM

#FIXME! The core URI auth code is tested, but not the use in this module
if URI_SPACE == '*':
    #Allow all URI access
    ALLOWED = [(True, True)]
else:
    ALLOWED = []
    for baseuri in URI_SPACE:
        #dAd a rule that permits URIs starting with this URISPACE item
        #FIXME: Technically should normalize uri and base, but this will work for most cases
        ALLOWED.append((lambda uri, base=baseuri: uri.startswith(base), True))
    

@simple_service('POST', XSLT_SERVICE_ID, 'akara.xslt')
def akara_xslt(body, ctype, **params):
    '''
    @xslt - URL to the XSLT transform to be applied
    all other query parameters are passed ot the XSLT processor as top-level params
    
    Sample request:
    curl --request POST --data-binary "@foo.xml" --header "Content-Type: application/xml" "http://localhost:8880/akara.xslt?@xslt=http://hg.akara.info/amara/trunk/raw-file/tip/demo/data/identity.xslt"
    '''
    if "@xslt" in params:
        akaraxslttransform = params["@xslt"]
    else:
        if not DEFAULT_TRANSFORM:
            raise ValueError('XSLT transform required')
        akaraxslttransform = DEFAULT_TRANSFORM
    restricted_resolver = irihelpers.resolver(authorizations=ALLOWED)
    #Using restricted_resolver should forbid Any URI access outside the specified "jails"
    #Including access through imports and includes
    body = inputsource(body, resolver=restricted_resolver)
    akaraxslttransform = inputsource(akaraxslttransform, resolver=restricted_resolver)
    result = transform(body, akaraxslttransform)

    response.add_header("Content-Type", result.parameters.media_type)
    return result 


@simple_service('POST', XPATH_SERVICE_ID, 'akara.xpath', 'text/xml')
def akara_xpath(body, ctype, **params):
    '''
    select - XPath expression to be evaluated against the document
    tidy - 'yes' to tidy HTML, or 'no'

    Sample request:
    curl --request POST --data-binary "@foo.xml" --header "Content-Type: application/xml" "http://localhost:8880/akara.xpath?select=/html/head/title&tidy=yes"
    '''
    if params.get("tidy") == 'yes':
        doc = html.parse(body)
    else:
        doc = amara.parse(body)
    result = simplify(doc.xml_select(params['select'].decode('utf-8')))
    return str(result)




# -*- coding: iso-8859-1 -*-
"""
MoinMoin - "text/xml" Formatter using Amara 2.x

@copyright: 2010 Uche Ogbuji <uche@ogbuji.net>
@license: Apache 2.0
"""
#Goes in plugin/formatter under moin data dir

from MoinMoin.formatter import FormatterBase
from MoinMoin import config
from MoinMoin.Page import Page

from amara import tree
from amara.writers.struct import structwriter, E, NS, ROOT, RAW
from amara.bindery.html import markup_fragment
from amara.lib import inputsource
from amara.lib import U

#This check ensures it's OK to just use Amara's U function directly
if config.charset != 'utf-8':
    #Note: we should never get here unless the user has messed up their config
    raise RuntimeError('Only UTF-8 encoding supported by Moin, and thus by us')
    

class Formatter(FormatterBase):
    """
    Send XML data.
    """
    def __init__(self, request, **kw):
        FormatterBase.__init__(self, request, **kw)
        self._current_depth = 1
        self._base_depth = 0
        self.in_pre = 0
        self._doc = tree.entity()
        self._curr = self._doc
        #self._writer = structwriter(indent=u"yes", encoding=config.charset)
        return

    def macro(self, macro_obj, name, args, markup=None):
        #Macro response are (unescaped) markup.  Do what little clean-up we camn, and cross fingers
        output = FormatterBase.macro(self, macro_obj, name, args, markup=markup)
        #response is Unicode
        if output:
            output_body = markup_fragment(inputsource.text(output.encode(config.charset)))
            #print "macro 2", repr(output)
            self._curr.xml_append(output_body)
        return ''

    def startDocument(self, pagename):
        self._curr = tree.element(None, u's1')
        self._curr.xml_attributes[None, u'title'] = U(pagename)
        self._doc.xml_append(self._curr)
        return ''

    def endDocument(self):
        #Yuck! But Moin seems to insist on Unicode object result (see MoinMoin.parser.text_moin_wiki.Parser.scan)
        #print "endDocument", repr(self._doc.xml_encode(encoding=config.charset).decode(config.charset))
        return U(self._doc.xml_encode(encoding=config.charset))

    def _elem(self, name, on, **kw):
        if on:
            e = tree.element(None, name)
            self._curr.xml_append(e)
            self._curr = e
        elif name == self._curr.xml_local:
            self._curr = self._curr.xml_parent
        return

    def lang(self, on, lang_name):
        self._elem(u'div', on)
        if on:
            self._curr.xml_attributes[None, u'lang'] = U(lang_name)
        return ''

    def sysmsg(self, on, **kw):
        self._elem(u'sysmsg', on)
        return ''

    def rawHTML(self, markup):
        #output = htmlparse(markup).html.body.xml_encode() if markup else ''
        if markup:
            body = markup_fragment(inputsource.text(markup))
            for child in body.xml_children:
                self._curr.xml_append(child)
        #self._curr.xml_append(tree.text(output.decode(config.charset)))
        #print "rawHTML", htmlparse(markup).xml_encode()
        return ''

    def pagelink(self, on, pagename='', page=None, **kw):
        FormatterBase.pagelink(self, on, pagename, page, **kw)
        if page is None:
            page = Page(self.request, pagename, formatter=self)
        link_text = page.link_to(self.request, on=on, **kw)
        self._curr.xml_append(tree.text(U(link_text)))
        return ''

    def interwikilink(self, on, interwiki='', pagename='', **kw):
        self._elem(u'interwiki', on)
        if on:
            self._curr.xml_attributes[None, u'wiki'] = U(interwiki)
            self._curr.xml_attributes[None, u'pagename'] = U(pagename)
        return ''

    def url(self, on, url='', css=None, **kw):
        self._elem(u'jump', on)
        self._curr.xml_attributes[None, u'url'] = U(url)
        if css:
            self._curr.xml_attributes[None, u'class'] = U(css)
        return ''

    def attachment_link(self, on, url=None, **kw):
        self._elem(u'attachment', on)
        if on:
            self._curr.xml_attributes[None, u'href'] = U(url)
        return ''

    def attachment_image(self, url, **kw):
        self._elem(u'attachmentimage', on)
        if on:
            self._curr.xml_attributes[None, u'href'] = U(url)
        return ''

    def attachment_drawing(self, url, text, **kw):
        self._elem(u'attachmentimage', on)
        self._curr.xml_attributes[None, u'href'] = U(url)
        self._curr.xml_append(tree.text(U(text)))
        self._elem(u'attachmentimage', off)
        return ''

    def attachment_inlined(self, url, text, **kw):
        from MoinMoin.action import AttachFile
        import os
        _ = self.request.getText
        pagename, filename = AttachFile.absoluteName(url, self.page.page_name)
        fname = wikiutil.taintfilename(filename)
        fpath = AttachFile.getFilename(self.request, pagename, fname)
        ext = os.path.splitext(filename)[1]
        Parser = wikiutil.getParserForExtension(self.request.cfg, ext)
        if Parser is not None:
            try:
                content = file(fpath, 'r').read()
                # Try to decode text. It might return junk, but we don't
                # have enough information with attachments.
                content = wikiutil.decodeUnknownInput(content)
                colorizer = Parser(content, self.request, filename=filename)
                colorizer.format(self)
            except IOError:
                pass

        self.attachment_link(1, url)
        self.text(text)
        self.attachment_link(0)
        return ''

    def smiley(self, text):
        self._curr.xml_append(tree.text(U(text)))
        return ''

    def text(self, text, **kw):
        self._curr.xml_append(tree.text(text))
        return ''

    def rule(self, size=0, **kw):
        e = tree.element(None, u'br') # <hr/> not supported in stylebook
        e.xml_append(tree.text((u"-" * 78)))
        self._curr.xml_append(e)
        return ''

    def icon(self, type_):
        self._elem(u'icon', on)
        self._curr.xml_attributes[None, u'type'] = U(type_)
        self._elem(u'icon', off)
        return ''

    def strong(self, on, **kw):
        self._elem(u'strong', on)
        return ''

    def emphasis(self, on, **kw):
        self._elem(u'em', on)
        return ''

    def highlight(self, on, **kw):
        self._elem(u'strong', on)
        return ''

    def number_list(self, on, type=None, start=None, **kw):
        self._elem(u'ol', on)
        return ''

    def bullet_list(self, on, **kw):
        self._elem(u'ul', on)
        return ''

    def listitem(self, on, **kw):
        self._elem(u'li', on)
        return ''

    def code(self, on, **kw):
        self._elem(u'code', on)
        return ''

    def small(self, on, **kw):
        self._elem(u'small', on)
        return ''

    def big(self, on, **kw):
        self._elem(u'big', on)
        return ''

    def sup(self, on, **kw):
        self._elem(u'sup', on)
        return ''

    def sub(self, on, **kw):
        self._elem(u'sub', on)
        return ''

    def strike(self, on, **kw):
        self._elem(u'strike', on)
        return ''

    def preformatted(self, on, **kw):
        FormatterBase.preformatted(self, on)
        self._elem(u'source', on)
        return ''

    def paragraph(self, on, **kw):
        FormatterBase.paragraph(self, on)
        self._elem(u'p', on)
        return ''

    def linebreak(self, preformatted=1):
        e = tree.element(None, u'br')
        self._curr.xml_append(e)
        return ''

    def heading(self, on, depth, id=None, **kw):
        # remember depth of first heading, and adapt current depth accordingly
        if not self._base_depth:
            self._base_depth = depth
        depth = max(depth + (2 - self._base_depth), 2)
        name = u's%i'%depth
        if on:
            found = None
            parent_depth = depth-1
            while not found:
                found = self._curr.xml_select(u'ancestor-or-self::' + u's%i'%(parent_depth))
                parent_depth -= 1
                if found:
                    break
            #print name, found
            self._curr = found[0]
            e = tree.element(None, name)
            id = U(id) if id else u''
            e.xml_attributes[None, u'title'] = id
            e.xml_attributes[None, u'id'] = id
            self._curr.xml_append(e)
            self._curr = e
            e = tree.element(None, u'title')
            self._curr.xml_append(e)
            self._curr = e
        else:
            parent = self._curr.xml_parent
            if self._curr.xml_local == u'title':
                parent.xml_remove(self._curr)
            self._curr = parent
        return ''

    def table(self, on, attrs={}, **kw):
        self._elem(u'table', on)
        return ''

    def table_row(self, on, attrs={}, **kw):
        self._elem(u'tr', on)
        return ''

    def table_cell(self, on, attrs={}, **kw):
        self._elem(u'td', on)
        return ''

    def anchordef(self, id):
        e = tree.element(None, u'anchor')
        self._curr.xml_append(e)
        self._curr.xml_attributes[None, u'id'] = U(id)
        return ''

    def anchorlink(self, on, name='', **kw):
        self._elem(u'link', on)
        if on:
            id = kw.get('id', None)
            if id:
                self._curr.xml_attributes[None, u'id'] = U(id)
            self._curr.xml_attributes[None, u'anchor'] = U(name)
        return ''

    def underline(self, on, **kw):
        return self.strong(on) # no underline in StyleBook

    def definition_list(self, on, **kw):
        self._elem(u'gloss', on)
        return ''

    def definition_term(self, on, compact=0, **kw):
        self._elem(u'label', on)
        return ''

    def definition_desc(self, on, **kw):
        self._elem(u'item', on)
        return ''

    def image(self, src=None, **kw):
        e = tree.element(None, u'img')
        self._curr.xml_append(e)
        valid_attrs = ('src', 'width', 'height', 'alt', 'title')
        kw.update({'src': src})
        for key, value in kw.items():
            if key in valid_attrs:
                self._curr.xml_attributes[None, U(key)] = U(value)
        return ''

    def transclusion(self, on, **kw):
        # TODO, see text_html formatter
        return ''

    def transclusion_param(self, **kw):
        # TODO, see text_html formatter
        return ''

    def code_area(self, on, code_id, code_type='code', show=0, start=-1, step=-1, msg=None):
        self._elem(u'codearea', on)
        if on:
            self._curr.xml_attributes[None, u'id'] = U(code_id)
        return ''

    def code_line(self, on):
        self._elem(u'codeline', on)
        return ''

    def code_token(self, on, tok_type):
        self._elem(u'codetoken', on)
        if on:
            self._curr.xml_attributes[None, u'type'] = U(tok_type)
        return ''



# This tests Akara through a running web server, started on an unused port.
# Some of the tests will require Akara to connect to other machines, which
# means this must be run with a working network.

# NOTE: you can test against a running server by setting AKARA_TEST_SERVER
# Here's one way to use a server on a different port on localhost:
#   env AKARA_TEST_SERVER=localhost:49658 nosetests test_demo_modules.py
# I do this to get around the server startup costs when developing/debugging tests.

from server_support import server

import urllib, urllib2
from urllib2 import urlopen
import os
from email.utils import formatdate

from amara import bindery
from amara.lib import iri
from amara.tools import atomtools

RESOURCE_DIR = os.path.join(os.path.dirname(__file__), "resource")

# the index "/"
def test_index():
    url = server()
    s = urlopen(url).read()
    assert "http://purl.org/xml3k/akara/services/registry" in s
    assert "akara.atom.json" in s

def test_index_search():
    url = server() + "?service=http://purl.org/xml3k/akara/services/registry"
    s = urlopen(url).read()
    assert "http://purl.org/xml3k/akara/services/registry" in s
    assert "akara.atom.json" not in s
    

# atomtools.py
def test_atom_json():
    from amara.thirdparty import json
    url = server() + "akara.atom.json?url=http://zepheira.com/feed/atom/"
    response = urlopen(url)
    results = json.load(response)
    items = results["items"]
    for item in items:
        assert "title" in item
test_atom_json.network=True

def test_aggregate_atom():
    url = server() + "akara.aggregate.atom"
    response = urlopen(url)
    doc = bindery.parse(response, model=atomtools.FEED_MODEL)
    assert str(doc.feed.title[0]) == "Feed me!", str(doc.feed.title[0])
    assert len(doc.feed.entry) == 3, len(doc.feed.entry)

def test_webfeedjson():
    from amara.thirdparty import json
    import json
    url = server() + "akara.webfeed.json?url=http://feeds.delicious.com/v2/rss/recent%3Fmin=1%26count=15"
    response = urlopen(url)
    results = json.load(response)
    print results

# calweb.py

# Frankly, this module doesn't seem that useful, so I'll only check
# to see that I get a response and that it contains a "akaraCalendarToday"
def test_calendar():
    url = server() + "akara.calendar"
    s = urlopen(url).read()
    assert "akaraCalCalendarToday" in s


# icaltools.py

def test_ical2json():
    from amara.thirdparty import json

    ical_filename = os.path.join(RESOURCE_DIR, "icalendar_test.ics")
    url = server() + "ical.json"

    req = urllib2.Request(url)
    req.add_header('Content-Type', 'text/calendar')

    data = open(ical_filename).read()

    response = urllib2.urlopen(req, data)
    results = json.load(response)

    items = results["items"]
    assert len(items) == 2
    assert items[0]["summary"] == "Bastille Day Party"
    assert items[1]["summary"] == "Akara test"

# luckygoogle.py

def test_luckygoogle():
    url = server() + "akara.luckygoogle?q=google"
    response = urlopen(url)
    s = response.read()
    assert s == "http://www.google.com/\n", repr(s)


# markuptools.py
def test_akara_twc():
    url = server() + "akara.twc.xml?max=5" # max 5 words in the result

    req = urllib2.Request(url)
    req.add_header('Content-Type', 'application/xml')

    data = "<a>one two <b>three four </b><c>five <d>six seven</d> eight</c> nine</a>"
    
    result = urllib2.urlopen(req, data).read()
    assert "one" in result, repr(result)
    assert "five" in result, repr(result)
    assert "six" not in result, repr(result)
    assert "nine" not in result, repr(result)

def test_akara_twc_html():
    url = server() + "akara.twc.xml?html=yes&max=11" # max 11 words

    req = urllib2.Request(url)
    req.add_header('Content-Type', 'application/xml')

    data = ("<html><head/><body>" + " ".join(map(str, range(510))) + "</body></html>")
    
    result = urllib2.urlopen(req, data).read()
    assert "7 8 9 10" in result, repr(result)
    assert "11" not in result, repr(result)

# method_dispatcher.py

def test_method_dispatcher_get():
    url = server() + "vikings?word=spam"
    response = urlopen(url)
    s = response.read()
    assert "spam-itty spam" in s, s

def test_method_dispatcher_get2():
    url = server() + "vikings?word=Spam"
    response = urlopen(url)
    s = response.read()
    assert "Spam-itty Spam" in s, s

def test_method_dispatcher_post():
    url = server() + "vikings"
    response = urlopen(url, "")
    s = response.read()
    assert s == "That was interesting.\n", s



# moin2atomentries.py - incomplete? Couldn't figure out what to test.

# moincms.py - the commented example fails, couldn't figure out what to test.

# moinrest - instructions didn't work, couldn't figure out what to test.

# oaitools - couldn't get it to work, couldn't figure out how to make it work, hence no tests.

# rdfatools.py

def test_rdfa2json():
    from amara.thirdparty import json
    url = server() + "akara.rdfa.json?url=http://zepheira.com/"
    results = json.load(urllib2.urlopen(url))
    for item in results["items"]:
        if "canonical" in item:
            assert "zepheira.com" in item["canonical"]
            break
    else:
        raise AssertionError("Could not find 'canonical'")


def test_rdfa2json_with_date():
    from amara.thirdparty import json
    url = server() + "akara.rdfa.json?url=http://www.myspace.com/parishilton"
    results = json.load(urllib2.urlopen(url))
    for item in results["items"]:
        if "canonical" in item:
            assert True
            break
    else:
        raise AssertionError("Could not find myspace:lastLogin")

# static.py

def test_static():
    # Pull up some static files
    url = server() + "resource/atom/entry1.atom"
    response = urllib2.urlopen(url)
    assert response.code == 200
    assert response.headers["Content-Type"] == "application/atom+xml", response.headers["Content-Type"]
    s = response.read()
    assert "Poster Boy @ Flickr" in s

    url = server() + "static/README"
    s = urllib2.urlopen(url).read()
    assert "SECRET MESSAGE" in s

    # Check that that leading "/" is trimmed
    url = server() + "//static////README"
    s = urllib2.urlopen(url).read()
    assert "SECRET MESSAGE" in s

def _http_failure(url, code):
    try:
        urllib2.urlopen(url).read()
        raise AssertionError("unexpected success")
    except urllib2.HTTPError, err:
        assert err.code == code, (err.code, code)

def test_static_missing_file():
    url = server() + "static/missing_file"
    _http_failure(url, 404)

# Interesting. After the upgrade to the most recent paste.httpserver
# this test fails with a 404 error instead of a 401 error. The
# new httpserver does a posixpath.normpath and converts the
# "/static/../widefinder_100.apache_log" ... to
# "/widefider_100.apache_log", which means I get a 404 error.
def test_static_bad_relative_file():
    url = server() + "static/../widefinder_100.apache_log"
    err = _http_failure(url, 404)


def test_static_unauthorized():
    url = server() + "static"
    _http_failure(url, 401)


# The simple code in the server only checks to see
# if the timestamp has changed.
_readme = os.path.join(RESOURCE_DIR, "static", "README")
_modified_since = formatdate(os.stat(_readme).st_mtime, usegmt=True)

def test_static_last_modified():
    url = server() + "static/README"
    req = urllib2.Request(url)
    req.add_header('If-Modified-Since', _modified_since)
    try:
        response = urllib2.urlopen(req)
        raise AssertionError("testing shows that this path isn't taken")
    except urllib2.HTTPError, err:
        assert err.code == 304, err.code

# statstools - needs R installed

# svntools - needs SVN installed

# unicodetools.py

def test_charbyname():
    url = server() + "akara.unicode.charbyname?name=DOUBLE+DAGGER"
    s = urllib2.urlopen(url).read()
    assert s == u"\N{DOUBLE DAGGER}".encode("utf-8")

def test_charbyname_missing():
    url = server() + "akara.unicode.charbyname?name=ETAOIN+SHRDLU"
    s = urllib2.urlopen(url).read()
    assert s == ""

def test_charsearch():
    url = server() + "akara.unicode.search?q=DAGGER"
    doc = bindery.parse(urllib2.urlopen(url))
    names = set()
    see_alsos = set()
    for child in doc.xml_select(u"characters/character"):
        names.add(child.name)
        see_alsos.add(child.see_also)
    assert names == set(["DAGGER", "DOUBLE DAGGER"]), names
    assert see_alsos == set(
        ["http://www.fileformat.info/info/unicode/char/2020/index.htm",
         "http://www.fileformat.info/info/unicode/char/2021/index.htm"]), see_alsos


# wwwlogviewer.py

_apache_query_data = open(os.path.join(RESOURCE_DIR, "widefinder_100.apache_log")).read()
def _make_log2json_request(query_args):
    from amara.thirdparty import json
    url = server() + "akara.wwwlog.json" + query_args
    req = urllib2.Request(url)
    req.add_header("Content-Type", "text/plain")
    response = urllib2.urlopen(req, _apache_query_data)
    return json.load(response)

def test_wwwlog2json():
    results = _make_log2json_request("")
    items = results["items"]
    assert len(items) == 200, len(items)
    assert items[0]["origin"] == "host-24-225-218-245.patmedia.net"
    assert items[-1]["timestamp"] == "2006-10-01T06:36:01-07:00"

def test_wwwlog2json_maxrecords():
    results = _make_log2json_request("?maxrecords=10")
    items = results["items"]
    assert len(items) == 10, len(items)
    assert items[0]["origin"] == "host-24-225-218-245.patmedia.net"
    assert items[-1]["timestamp"] == "2006-10-01T06:33:55-07:00", items[-1]["timestamp"]

def test_wwwlog2json_maxrecords_large():
    # Size is greater than the number of records
    results = _make_log2json_request("?maxrecords=1000")
    items = results["items"]
    assert len(items) == 200, len(items)

def test_wwwlog2json_nobots():
    results = _make_log2json_request("?nobots=1")
    items = results["items"]
    assert len(items) == 183, len(items)

# xslt.py
XML_DATA = open(os.path.join(RESOURCE_DIR, "xslt_spec_example.xml")).read()
XSLT_URL = iri.os_path_to_uri(os.path.join(RESOURCE_DIR, "xslt_spec_example.xslt"))
def test_xslt():
    url = server() + "akara.xslt?" + urllib.urlencode({"@xslt": XSLT_URL})
    req = urllib2.Request(url)
    req.add_header("Content-Type", "text/xml")
    response = urllib2.urlopen(req, XML_DATA)

    doc = bindery.parse(response)
    assert str(doc.html.head.title) == "Document Title", repr(str(doc.html.head.title))
    
if __name__ == "__main__":
    raise SystemExit("Use nosetests")



from server_support import server
import urllib2
from urllib2 import urlopen
from collections import defaultdict

import amara

# check that the servers restart after 10 calls
# (MaxRequestsPerServer = 5 * MaxServers = 5 means no more than 25 calls)

_legal_counts = set("0 1 2 3 4".split())
def test_restart():
    url = server() + "test_get_call_count"
    
    pid_counts = defaultdict(list)
    for i in range(50):
        result = urlopen(url).read()
        count, pid = result.split()
        pid_counts[pid].append(count)

    items = sorted(pid_counts.items())
    for pid, counts in items:
        print pid, counts

    print "NOTE: this is only valid for special configuration setups"
    for pid, counts in items:
        counts_set = set(counts)
        assert len(counts) == len(counts_set), repr(counts)
        for count in counts:
            assert count in _legal_counts, count

def test_index():
    url = server()
    xml = urlopen(url).read()
    print xml
    assert "This is an internal class" not in xml
    assert ("<path>test_echo_simple_get</path>"
            "<description>This echos a GET request, including the QUERY_STRING</description>") in xml
    assert "<path>test_multimethod</path><description>SHRDLU says 'QWERTY'</description>" in xml

def test_index_search():
    url = server() + "?service=http://example.com/test_echo"
    xml = urlopen(url).read()
    assert xml.startswith('''<?xml version="1.0" encoding="utf-8"?>
<services><service '''), repr(xml[:100])
    assert ('<service ident="http://example.com/test_echo">'
            '<path>test_echo_simple_get</path>'
            '<description>This echos a GET request, including the QUERY_STRING</description>'
            '</service>') in xml
    
    assert ('<service ident="http://example.com/test_echo">'
            '<path>test_echo_simple_get2</path>'
            '<description>Hi test_server.py!</description>'
            '</service>') in xml
    assert xml.count("<service ") == 2, xml.count("<service ")

def test_index_search_no_hits():
    url = server() + "?service=http://example.com/test_echo_werwrwerqwerqwerqwerqwerqwer"
    xml = urlopen(url).read()
    assert xml == ('<?xml version="1.0" encoding="utf-8"?>\n'
                   '<services/>'), repr(xml)

def test_404_error_message():
    url = server() + "this_does_not_exist/I_mean_it/Anybody_want_a_peanut?"
    try:
        urlopen(url)
        raise AssertionError("that URL should not be present")
    except urllib2.HTTPError, err:
        assert err.code == 404
        assert err.headers["Content-Type"] == "text/html", err.headers["Content-Type"]
        tree = amara.parse(err.fp, standalone=True)

def test_405_error_message():
    url = server()
    try:
        f = urlopen(url, "The server ignores this text")
        raise AssertionError("/ is not supposed to allow a POST")
    except urllib2.HTTPError, err:
        assert err.code == 405, err.code
        assert err.headers["Content-Type"] == "text/html", err.headers["Content-Type"]
        tree = amara.parse(err.fp, standalone=True)

def test_405_error_message_mega(n=100):
    import time
    t1=time.time()
    for i in range(n):
        test_405_error_message()
    t2=time.time()
    # This catches an error in the error handling code where I did
    # not parse in standalone mode, causing a 0.75s network load.
    if n / (t2-t1+0.00001) < 2:
        raise AssertionError("Should be able to handle more than 2 failure requests/seconds")

if __name__ == "__main__":
    def server():
        #return "http://192.168.2.101:8880/"
        return "http://localhost:8880/"
    test_405_error_message_mega(1000)



from __future__ import with_statement
import tempfile
import shutil
import os
import sys
from cStringIO import StringIO

from akara import commandline, read_config

class Config(object):
    def __init__(self, config_root):
        self.config_filename = os.path.join(config_root, "pid_test.ini")
        self.pid_filename = os.path.join(config_root, "pid.txt")
    def save_pid(self, text):
        f = open(self.pid_filename, "w")
        try:
            f.write(text)
        finally:
            f.close()

# 'akara stop' and 'akara restart' essentially just call get_pid()
# plus do one extra call. The os.kill calls are tested manually.


def tmpdir(func):
    def wrapper():
        dirname = tempfile.mkdtemp(prefix="akara_test_")
        try:
            func(dirname)
        finally:
            shutil.rmtree(dirname)
    wrapper.__name__ = func.__name__
    return wrapper

@tmpdir
def test_get_pid(config_root):
    config = Config(config_root)
    f = open(config.config_filename, "w")
    try:
        f.write("class Akara:\n  PidFile = %r\nMODULES=[]\n" % config.pid_filename)
    finally:
        f.close()
    try:
        commandline.get_pid(config)
        raise AssertionError("But the file does not exist!")
    except SystemExit, err:
        assert "Could not open Akara PID file" in str(err), err
        assert config.pid_filename in str(err), err

    config.save_pid("")
    try:
        commandline.get_pid(config)
    except SystemExit, err:
        assert "Empty Akara PID file" in str(err), err
        assert config.pid_filename in str(err), err

    config.save_pid("hello\n")
    try:
        commandline.get_pid(config)
    except SystemExit, err:
        assert "does not contain a PID" in str(err), err
        assert config.pid_filename in str(err), err

    config.save_pid("123\n")
    pid = commandline.get_pid(config)
    assert pid == 123


class CaptureStdout(object):
    def __init__(self):
        self.io = StringIO()
    def __enter__(self):
        self.io.reset()
        self.io.truncate()
        self.stdout = sys.stdout
        sys.stdout = self.io
    def __exit__(self, *args):
        sys.stdout = self.stdout
        self.content = self.io.getvalue()
        

@tmpdir
def test_status(config_root):
    config = Config(config_root)
    capture = CaptureStdout()
    with capture:
        try:
            commandline.status(config)
            raise AssertionError("should not get here")
        except SystemExit:
            pass
    assert "Could not open Akara configuration file" in capture.content
    assert config.config_filename in capture.content
    assert "Error log file" not in capture.content

    f = open(config.config_filename, "w")
    f.write("class Akara: PidFile = %r\n" % config.pid_filename)
    f.close()
    with capture:
        commandline.status(config)
    assert "PID file" in capture.content
    assert "PID file does not exist" in capture.content
    assert "Akara is not running" in capture.content
    assert "Cannot open PID file" not in capture.content

    config.save_pid("Scobby-doo!\n")
    with capture:
        try:
            commandline.status(config)
            raise AssertionError("where was the exit? %r" % capture.io.getvalue())
        except SystemExit:
            pass
    assert "Unable to parse the PID" in capture.content
    assert "Scobby-doo" in capture.content

    os.chmod(config.pid_filename, 0)
    with capture:
        try:
            commandline.status(config)
            raise AssertionError("where was the exit? %r" % capture.io.getvalue())
        except SystemExit:
            pass
    assert "*** Cannot open PID file" in capture.content


    os.chmod(config.pid_filename, 0644)
    my_pid = str(os.getpid())
    config.save_pid(my_pid)
    with capture:
        commandline.status(config)

    assert ("PID is %s and there is a process" % my_pid) in capture.content
    assert "Akara is running" in capture.content

    # I can't think of a good way to test for a PID which does not exist.
    # That test is done manually.

@tmpdir
def test_setup_config_file(config_root):
    config_file = os.path.join(config_root, "blah_subdir", "test_config.ini")

    assert not os.path.exists(config_file)

    capture = CaptureStdout()
    with capture:
        commandline._setup_config_file(config_file)
    assert "Copying reference configuration file" in capture.content
    assert "Creating directory" in capture.content
    assert "blah_subdir" in capture.content

    assert os.path.exists(config_file)
    s = open(config_file).read()
    assert "class Akara" in s
    assert " Listen" in s

    with capture:
        commandline._setup_config_file(config_file)
    assert "Configuration file already exists" in capture.content

@tmpdir
def test_setup(config_root):
    config = Config(config_root)
    capture = CaptureStdout()

    f = open(config.config_filename, "w")
    f.write("class Akara: ConfigRoot = %r\n" % config_root)
    f.close()

    with capture:
        commandline.setup(config)

    assert "Created error log directory" in capture.content
    assert "Access log directory exists" in capture.content
    assert "PID file directory exists" in capture.content, capture.content
    assert "Created extension modules directory" in capture.content
    
    assert os.path.exists(os.path.join(config_root, "logs"))
    assert os.path.exists(os.path.join(config_root, "modules"))

@tmpdir
def test_log_rotate(config_root):
    config = Config(config_root)
    capture = CaptureStdout()
    error_log_filename = os.path.join(config_root, "testing.log")
    
    def find_backup_logs():
        return [name for name in os.listdir(config_root)
                   if name.startswith("testing.log.")]
    
    with open(config.config_filename, "w") as f:
        f.write("class Akara:\n  ConfigRoot = %r\n  ErrorLog=%r\n" %
                (config_root, error_log_filename))

    # No log file present
    with capture:
        commandline.main(["akara", "-f", config.config_filename, "rotate"])
    assert "No log file"

    MESSAGE = "It was the best of times it was the worst of times.\n"
    with open(error_log_filename, "w") as f:
        f.write(MESSAGE)

    # Existing log file is present. Rotate
    with capture:
        commandline.main(["akara", "-f", config.config_filename, "rotate"])
    assert "testing.log.2" in capture.content, capture.content
    assert "Rotated log" in capture.content, capture.content

    filenames = find_backup_logs()
    assert len(filenames) == 1, ("should have one backup", filenames)

    # Check that the content rotated
    content = open(os.path.join(config_root, filenames[0])).read()
    assert content == MESSAGE, (content, MESSAGE)

    # The log file should not be present
    assert not os.path.exists(error_log_filename)
    MESSAGE = "When shall we three meet again?\n"
    with open(error_log_filename, "w") as f:
        f.write(MESSAGE)

    # And rotate again. Should now have two backups
    commandline.main(["akara", "-f", config.config_filename, "rotate"])    
    filenames = find_backup_logs()
    assert len(filenames) == 2, ("should have two backups", filenames)

    assert not os.path.exists(error_log_filename)



"""Information about the incoming request

The available functions and data elements are:

  environ - the corresponding WSGI 'environ' variable
  akara_config - a ConfigParser containing the data from the Akara configuration file

The following are defined if the Akara handler comes from an extension module
  module_name - the name of the extension module
  module_config - the ConfigParser section for the current extension module

These should all be treated as read-only constants.

"""
import os as _os

environ = None
akara_config = None

module_name = None


from UserDict import DictMixin as _DictMixin
class _ModuleConfig(_DictMixin):
#    # Implement an interface like ConfigParser but only for this section
#    def get(self, option):
#        return akara_config.get(module_name, option)
#    def getint(self, option):
#        return akara_config.getint(module_name, option)
#    def getfloat(self, option):
#        return akara_config.getfloat(module_name, option)
#    def getboolean(self, option):
#        return akara_config.getboolean(module_name, option)
#
#    def items(self):
#        return akara_config.items(module_name)
#
#    def set(self, option, value):
#        return akara_config.set(module_name, option, value)
#    def remove_option(option):
#        return akara_config.remove_option(module_name, option)
    
    # Implement a dictionary-like interface
    def __getitem__(self, key):
        return akara_config.get(module_name, key)
    def __setitem__(self, key, value):
        return akara_config.set(module_name, key, value)
    def __delitem__(self, key):
        return akara_config.remove_option(module_name, key)
    def keys(self):
        return akara_config.options(module_name)

module_config = _ModuleConfig()


# Eventually add cookie support?



# -*- encoding: utf-8 -*-
'''
PubSubHubbub

See also:

http://code.google.com/p/pubsubhubbub/
http://pubsubhubbub.googlecode.com/svn/trunk/pubsubhubbub-core-0.3.html

Requires a configuration section, for example:

[pubsubhubbub]

entries = /path/to/entry/files/*.atom
feed_envelope = <feed xmlns="http://www.w3.org/2005/Atom"><title>My feed</
title><id>http://example.com/myfeed</id></feed>

'''

import sys
from datetime import datetime, timedelta
import cgi
import sqlite3
from itertools import dropwhile

import amara
from amara import bindery
from amara.tools import atomtools
from amara.thirdparty import httplib2
from amara.lib.util import first_item

from akara.services import simple_service
from akara import request, response
from akara import logger

#akara.util.pubsubhubbub provides tools for creating subscribers

DBFILE = AKARA.module_config['dbfile']

#http://copia.posterous.com/quotidie-eliot-milton-and-ars-versificandi

def ensuredb():
    logger.debug('DBFILE: ' + repr(DBFILE))
    db = sqlite3.connect(DBFILE)
    try:
        db.execute("select count(*) from subscription")
    except sqlite3.OperationalError:
        # Create table
        #db.execute('''create table subscriber
        #(topic text, callback text, added timestamp)''')
        db.execute('''create table subscription
        (callback text, topic text, added timestamp)''')
        #(topic text, latlong text, city text, state text, country text, updated timestamp)''')
        db.commit()
    return db


def dict2obj(d):
    '''
    >>> p = dict2obj({'spam': 1, 'eggs': 2, 'bacon': 0})
    >>> p.spam
    1
    >>> p.eggs
    2
    '''
    #FIXME: use Bindery rules for handling problem dict keys
    class oclass(object): pass
    o = oclass()
    for k, v in d.iteritems():
        setattr(o, k, v)
    return o


SERVICE_ID = 'http://purl.org/akara/services/demo/pubsubhubbub.sub'
@simple_service('POST', SERVICE_ID, 'akara.pubsubhubbub.sub')
def pubsubhubbub_sub(body, ctype):
    '''
    Sample requests:
    * curl --data "mode=subscribe&callback=http%3A%2F%2Flocalhost%3A8880%2Fpshbtestcb&topic=http%3A%2F%2Flocalhost%3A8880%2Fpshbtesttop" "http://localhost:8880/akara.pubsubhubbub.sub"

    sqlite3 /tmp/pubsubhubbub.db 'select * from subscription;'

    '''
    
    '''
    import httplib, urllib
    params = urllib.urlencode({'spam': 1, 'eggs': 2, 'bacon': 0}) #-> application/x-www-form-urlencoded
    '''
    logger.debug('parsed: ' + repr(dict(cgi.parse_qsl(body))))
    hub = dict2obj(dict(cgi.parse_qsl(body)))
    db = ensuredb()
    if hub.mode == 'unsubscribe':
        resp = db.execute("delete from subscription where callback=? and topic=?", (hub.callback, hub.topic))
        logger.debug('accepted_imts: unsubscribe' + repr(resp))
        db.commit()
    elif hub.mode == 'subscribe':
        resp = db.execute("insert into subscription values (?, ?, ?)", (hub.callback, hub.topic, datetime.now(),))
        logger.debug('accepted_imts: subscribe' + repr(resp))
        db.commit()

    #hub.verify
    #hub.lease_seconds
    #hub.secret
    #hub.verify_token
    response.code = 204 #Or 202 for deferred verification
    return ''


@simple_service('POST', SERVICE_ID, 'akara.pubsubhubbub.hub')
def pubsubhubbub_hub(body, ctype):
    '''
    Sample requests:
    * curl --data "mode=subscribe&callback=http%3A%2F%2Flocalhost%3A8880%2Fpshbtestcb&topic=http%3A%2F%2Flocalhost%3A8880%2Fpshbtesttop" "http://localhost:8880/akara.pubsubhubbub.hub"
    * curl --data "mode=publish&callback=http%3A%2F%2Flocalhost%3A8880%2Fpshbtestcb&topic=http%3A%2F%2Flocalhost%3A8880%2Fpshbtesttop" "http://localhost:8880/akara.pubsubhubbub.hub"
    '''
    logger.debug('parsed: ' + repr(dict(cgi.parse_qsl(body))))
    hub = dict2obj(dict(cgi.parse_qsl(body)))
    db = ensuredb()
    if hub.mode == 'subscribe':
        #Request from a subscriber to subscribe to a topic of interest
        resp = db.execute("insert into subscription values (?, ?, ?)", (hub.callback, hub.topic, datetime.now(),))
        logger.debug('accepted_imts: subscribe' + repr(resp))
        db.commit()

    if hub.mode == 'publish':
        #Ping notification from a publisher whose topics have changed.
        resp = db.execute("insert into subscription values (?, ?, ?)", (hub.callback, hub.topic, datetime.now(),))
        logger.debug('accepted_imts: subscribe' + repr(resp))
        db.commit()

    #hub.verify
    #hub.lease_seconds
    #hub.secret
    #hub.verify_token
    response.code = 204 #Or 202 for deferred verification
    return ''





"""
global_config holds the values of the Akara configuration parameters
from the [global] section of the configuration file.  The parameters
have the same names as those used in the settings parameter created
by the read_config.py.

Common configuration variables include:

server_address         : (host,port) of the Akara server
server_root            : public URL to the top-level of Akara
internal_server_root   : internal URL to the top-level of Akara
config_root            : Akara configuration root directory (e.g., ~/.local/lib/akara)
pid_file               : Location of the PID file
error_log              : Filename of the Akara error log
access_log             : Filename of the Akara access log
module_dir             : Akara module directory
module_cache           : Module cache directory
log_level              : Logging level
max_servers            : Maximum number of servers
min_spare_servers      : Min spare servers
max_spare_servers      : Max spare servers
max_requests_per_server: Max requests per server

The primary purpose of this module is to make configuration parameters
available to various library modules that make up the Akara core. 
"""

# Initially this module is defined as empty.   It is populated by the
# akara.run module which reads the global config file and obtains the
# values of the various configuration paramaters







import hashlib
import urllib2

from test_services import GET

from akara import pipeline

def test_pipeline_missing_stages():
    for stages in (None, [], ()):
        try:
            pipeline.register_pipeline("blah", stages=stages)
        except TypeError:
            pass
        else:
            raise AssertError("allowed missing stages: %r" % stages)

def test_flatten():
    result = list(pipeline._flatten_kwargs_values(dict(a=["1","2","3"])))
    assert result == [("a","1"), ("a","2"), ("a","3")], result
    result = list(pipeline._flatten_kwargs_values(dict(a=["1","2","3"], b="9")))
    result.sort()
    assert result == [("a","1"), ("a","2"), ("a","3"), ("b","9")], result

def test_stage_query_args():
    stage = pipeline.Stage("http://example.com", [("a", ["1", "2"]), ("b", "9")])
    assert stage.query_string == "a=1&a=2&b=9", stage.query_string

def test_stage_kwargs():
    stage = pipeline.Stage("http://example.com", a=["1", "2"], b="9")
    assert (stage.query_string == "a=1&a=2&b=9" or
            stage.query_string == "b=9&a=1&a=2"), stage.query_string
    
def test_stage_raw_query():
    stage = pipeline.Stage("http://example.com", query_string="=j")
    assert stage.query_string == "=j"

def test_stage_error_combinations():
    # Not allowed to mix inputs
    def t1():
        pipeline.Stage("http://example.com", [("a", "b")], query_string="=j")
    def t2():
        pipeline.Stage("http://example.com", [("a", "b")], a=3)
    def t3():
        pipeline.Stage("http://example.com", query_string="=j", a=3)

    for t in (t1, t2, t2):
        try:
            t()
        except TypeError:
            pass
        else:
            raise AssertionError("expected to fail")



def test_hash_encode():
    result = GET("hash_encode", data="This is a test")
    expected = hashlib.md5("secretThis is a test").digest().encode("base64")
    assert result == expected, (result, expected)

def test_hash_encode_rot13():
    result = GET("hash_encode_rot13", data="This is another test")
    expected = hashlib.md5("secretThis is another test").digest().encode("base64").encode("rot13")
    assert result == expected, (result, expected)

def test_get_hash():
    result = GET("get_hash")
    expected = hashlib.md5("Andrew").digest().encode("base64")
    assert result == expected, (result, expected)

def test_get_hash2():
    result = GET("get_hash", dict(text="Sara Marie"))
    expected = hashlib.md5("Sara Marie").digest().encode("base64")
    assert result == expected, (result, expected)

def test_broken_pipeline1():
    try:
        result = GET("broken_pipeline1")
        raise AssertionError("should not get here")
    except urllib2.HTTPError, err:
        assert err.code == 500
        msg = err.read()
        assert "Broken internal pipeline" in msg, msg

def test_broken_pipeline2():
    try:
        result = GET("broken_pipeline2", data="feed the pipeline")
        raise AssertionError("should not get here")
    except urllib2.HTTPError, err:
        assert err.code == 500, err.code
        msg = err.read()
        assert "Broken internal pipeline" in msg, msg

def test_registry_size():
    result = GET("test_count_registry")
    assert int(result) > 30, "What?! Did you remove elements from the registry?"



#akara.resource
"""
There is no superclass for an Akara resource.  It's any object with an id data mamber

self.policy - instance of L{akara.policy.manager}

"""

#__all__ = ['manager', 'standard_index']

CREATED = 'akara:created'
UPDATED = 'akara:updated'
CONTENT_LENGTH = 'akara:size'
CONTENT_TYPE = 'akara:type'

class resource(object):
    '''
    An analogue of a Web resource
    
    In effect it serves as a cache of the actual stored repository data
    
    Standard repository metadata:
    
    * Content type (internet media type)
    * Size
    * creation date
    * last mod date
    '''
    def __init__(self, rid, manager):
        self._manager = manager
        self.rid = rid
        self._metadata = None #Mixes/caches repository metadata and user metadata
        self._content = None
        return
        
    def __getitem__(self, name):
        return self._metadata[name]

    def _get_content(self):
        if self._content is None: self._sync()
        return self._content

    def _set_content(self, c):
        if self._content is None: self._sync()
        self._content = c

    content = property(_get_content, _set_content)

    @property
    def metadata(self):
        if self._metadata is None: self._sync()
        return self._metadata

    def _sync(self):
        '''
        Sync up this copy with the database
        '''
        drv = self._manager._driver
        content, self.metadata = drv.get_resource(self.rid)
        self.content = content.read()
        return


class manager(dict):
    """
    Maps aliases to IDs
    
    """
    #Manager itself is a very simple dict interface.  You would generally use a more specialized
    #object that includes the persistence layer
    #def __init__(self, input_dict={}):
    #    self.update(input_dict)

    def __init__(self, driver):
        self._driver = driver
        self.aliases = {}
        #FIXME: replace with MRU
        self._cache = {}
        return

    def lookup(self, name):
        '''
        Look up resource by ID
        '''
        rid = name
        if rid in self.aliases:
            rid = self.aliases[rid]
        if rid in self._cache:
            return elf._cache[rid]
        if self._driver.has_resource(rid):
            return resource(rid, self)
        else:
            raise RuntimeError('Resource not found: %s'%str(rid))
            #raise ResourceError
        return




# -*- encoding: utf-8 -*-
'''Demo of how to use Akara to talk with SVN via POST -- *INSECURE*

It includes an example of how to forward security information to a new
request.

This module takes an optional configuration section in akara.conf. The
default settings are:

class svntools:
    svn_commit = ["svn", "commit", "-m", "$MSG", "$FPATH"]
    svn_add = ["svn", "add", "$FPATH"]

where "$MSG" is replaced with the commit message (from the form
variable "msg") and "$FPATH" is replaced with the path to the file
under SVN.

NOTE: This demo module is *insecure*. Do not use in an untrusted
network. The module can be told to fetch an arbitrary URL and write
the result to an arbitrary file. It can also do SVN commits against
any accessible file in the file system.

'''
from __future__ import with_statement
import sys, time
import urllib, urllib2, urlparse
from subprocess import *
import cgi
from cStringIO import StringIO
from itertools import *
from contextlib import closing

from amara import _
from amara.lib.util import *

import akara
from akara.util import copy_auth
from akara.services import simple_service

Q_REQUIRED = _("The 'q' POST parameter is mandatory.")
SVN_COMMIT_CMD = akara.module_config().get('svn_commit', 'svn commit -m "%(msg)s" %(fpath)s')
SVN_ADD_CMD = akara.module_config().get('svn_add', 'svn add %(fpath)s')

SERVICE_ID = 'http://purl.org/akara/services/demo/svncommit'
@simple_service('POST', SERVICE_ID, 'akara.svncommit', 'text/plain')
def svncommit(body, ctype, **params):
    '''Commit a file. Can optionally populate the file contents from a given URL.

    The form parameters are:
      fpath - the name of the file to commit to SVN
      msg - the commit message
      q (optional) - fetch the given URL and save it to the specified file before commmiting
    
    The form must be POSTed as multipart/form-data. If the request includes
    the 'q' parameter then the new fetch will contain authentication 
    forward 
    
    Sample request:
      curl -F "msg=fixed a typo" -F fpath="/path/to/file" -F "q=http://example.org/content" http://localhost:8880/akara.svncommit

    '''
    body = StringIO(body)
    form = cgi.FieldStorage(fp=body, environ=WSGI_ENVIRON)
    #for k in form:
    #    print >> sys.stderr, (k, form.getvalue(k)[:100])
    q = form.getvalue('q')
    fpath = form.getvalue('fpath')
    msg = form.getvalue('msg')
    #assert_not_equal(q, None, msg=Q_REQUIRED)

    if q:
        handler = copy_auth(WSGI_ENVIRON, q)
        opener = urllib2.build_opener(handler) if handler else urllib2.build_opener()
        req = urllib2.Request(q)
        with closing(opener.open(req)) as resp:
            result = resp.read()
            ctype = dict(resp.info()).get('Content-Type')

        with closing(open(fpath, 'w')) as f:
            f.write(result)

    cmdline = SVN_COMMIT_CMD%{'msg': msg, 'fpath': fpath}
    print >> sys.stderr, 'Executing subprocess in shell: ', cmdline
    
    process = Popen(cmdline, stdout=PIPE, universal_newlines=True, shell=True)
    output, perr = process.communicate()

    return 'SVN commit OK\n'


URL_REQUIRED = _("The 'URL' POST parameter is mandatory.")

SERVICE_ID = 'http://purl.org/akara/services/demo/svncheckout'
@simple_service('GET', SERVICE_ID, 'akara.svncheckout')
def svncheckout(url=None):
    '''
    url - 
    
    Sample request:
    curl "http://localhost:8880/akara.svncheckout?url=http://zepheira.com"
    '''
    # Requires Python 2.6 or http://code.google.com/p/json/
    from amara.thirdparty import json
    ids = set()
    if url is None:
        raise AssertionError(URL_REQUIRED)
    with closing(urllib2.urlopen(url)) as resp:
        content = resp.read()
    resources = rdfaparse(content)
    return json.dumps({'items': resources}, indent=4)




# -*- encoding: utf-8 -*-
'''
See also:
 * [[http://www.eprints.org/software/xslt.php|"OAI2 to HTML XSLT Style Sheet"]]
'''

from __future__ import with_statement
import sys, time
import datetime
import urllib, urlparse
from cgi import parse_qs
from itertools import *

import amara
from amara import bindery
from amara import tree
from amara.writers.struct import *
from amara.bindery.model import examplotron_model, generate_metadata
from amara.lib import U

from amara.tools.atomtools import feed

from akara.services import simple_service

OAI_NAMESPACE = u"http://www.openarchives.org/OAI/2.0/"

#OAI-PMH verbs:
# * Identify
# * ListMetadataFormats
# * ListSets
# * GetRecord
# * ListIdentifiers
# * ListRecords


#Useful:
# http://www.nostuff.org/words/tag/oai-pmh/
# http://libraries.mit.edu/dspace-mit/about/faq.html
# http://wiki.dspace.org/index.php/OaiInstallations - List of OAI installations harvested by DSpace
#Examples:
# http://eprints.sussex.ac.uk/perl/oai2?verb=GetRecord&metadataPrefix=oai_dc&identifier=oai:eprints.sussex.ac.uk:67
# http://dspace.mit.edu/oai/request?verb=Identify
# http://dspace.mit.edu/oai/request?verb=GetRecord&metadataPrefix=oai_dc&identifier=oai:dspace.mit.edu:1721.1/5451

#Based on: http://dspace.mit.edu/oai/request?verb=GetRecord&metadataPrefix=oai_dc&identifier=oai:dspace.mit.edu:1721.1/5451
OAI_MODEL_XML = '''<?xml version="1.0" encoding="UTF-8"?>
<OAI-PMH xmlns="http://www.openarchives.org/OAI/2.0/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:o="http://www.openarchives.org/OAI/2.0/"
         xsi:schemaLocation="http://www.openarchives.org/OAI/2.0/ http://www.openarchives.org/OAI/2.0/OAI-PMH.xsd"
         xmlns:eg="http://examplotron.org/0/" xmlns:ak="http://purl.org/xml3k/akara/xmlmodel">
  <responseDate>2009-03-30T06:09:23Z</responseDate>
  <request verb="GetRecord" identifier="oai:dspace.mit.edu:1721.1/5451" metadataPrefix="oai_dc">http://dspace.mit.edu/oai/request</request>
  <GetRecord>
    <record ak:resource="o:header/o:identifier">
      <header>
        <identifier>oai:dspace.mit.edu:1721.1/5451</identifier>
        <datestamp ak:rel="local-name()" ak:value=".">2006-09-20T00:15:44Z</datestamp>
        <setSpec>hdl_1721.1_5443</setSpec>
      </header>
      <metadata>
        <oai_dc:dc xmlns:oai_dc="http://www.openarchives.org/OAI/2.0/oai_dc/" xmlns:dc="http://purl.org/dc/elements/1.1/" xsi:schemaLocation="http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd">
          <dc:creator ak:rel="local-name()" ak:value=".">Cohen, Joshua</dc:creator>
          <dc:date ak:rel="local-name()" ak:value=".">2004-08-20T19:48:34Z</dc:date>
          <dc:date>2004-08-20T19:48:34Z</dc:date>
          <dc:date>1991</dc:date>
          <dc:identifier ak:rel="'handle'" ak:value=".">http://hdl.handle.net/1721.1/5451</dc:identifier>
          <dc:description ak:rel="local-name()" ak:value=".">Cohen's Comments on Adam Przeworski's article "Could We Feed Everyone?"</dc:description>
          <dc:format>2146519 bytes</dc:format>
          <dc:format>application/pdf</dc:format>
          <dc:language>en_US</dc:language>
          <dc:publisher ak:rel="local-name()" ak:value=".">Politics and Society</dc:publisher>
          <dc:title ak:rel="local-name()" ak:value=".">"Maximizing Social Welfare or Institutionalizing Democratic Ideals?"</dc:title>
          <dc:type>Article</dc:type>
          <dc:identifier>Joshua Cohen, "Maximizing Social Welfare or Institutionalizing Democratic Ideals?"; Politics and Society, Vol. 19, No. 1</dc:identifier>
        </oai_dc:dc>
      </metadata>
    </record>
  </GetRecord>
</OAI-PMH>
'''

OAI_MODEL = examplotron_model(OAI_MODEL_XML)

ATOM_ENVELOPE = '''<?xml version="1.0" encoding="UTF-8"?>
<feed xmlns="http://www.w3.org/2005/Atom" xmlns:sd="http://kds.elsevier.com/datamodel/sciencedirect#" xmlns:os="http://a9.com/-/spec/opensearch/1.1/">
  <title>MIT DSpace</title>
  <id>http://dspace.mit.edu/</id>
</feed>
'''

SERVICE_ID = 'http://purl.org/akara/services/demo/oai.json'
@simple_service('GET', SERVICE_ID, 'akara.oai.atom', 'application/atom+xml')
def atomize_oai_record(endpoint=None, id=None):
    '''
    endpoint - the OAI request URL, e.g. http://dspace.mit.edu/oai/request
    id, e.g. the article ID, e.g. oai:dspace.mit.edu:1721.1/5451
    
    Sample request:
    curl "http://localhost:8880/akara.oai.atom?endpoint=http://dspace.mit.edu/oai/request&id=oai:dspace.mit.edu:1721.1/5451"
    '''
    if endpoint is None:
        raise ValueError('endpoint required')
    if id is None:
        raise ValueError('id required')
    qstr = urllib.urlencode({'verb' : 'GetRecord', 'metadataPrefix': 'oai_dc', 'identifier': id})
    url = endpoint + '?' + qstr
    doc = bindery.parse(url, model=OAI_MODEL)
    resources = metadata_dict(generate_metadata(doc))
    #print resources
    f = feed(ATOM_ENVELOPE)
    #f = feed(ATOM_ENVELOPE, title=resources['title'], id=resources['id'])
    #f.source.feed.xml_append(E((ATOM_NAMESPACE, u'link'), {u'rel': u'self', u'type': u'application/atom+xml', u'href': self_link.decode('utf-8')}))
    #f.source.feed.xml_append(E((ATOM_NAMESPACE, u'link'), {u'rel': u'search', u'type': u'application/opensearchdescription+xml', u'href': u'http://kds-kci.zepheira.com/sciencedirect.discovery'}))
    #f.source.feed.xml_append(E((ATOM_NAMESPACE, u'link'), {u'rel': u'alternate', u'type': u'text/xml', u'href': alt_link.decode('utf-8')}))
    #f.source.feed.xml_append(E((OPENSEARCH_NAMESPACE, u'Query'), {u'role': u'request', u'searchTerms': search_terms.decode('utf-8')}))
    #maxarticles = DEFAULT_MAX_RESULTS
    maxarticles = 3
    for record in islice(doc.OAI_PMH, 0, maxarticles):
        resource = unicode(resources[id])
        print resource
        authors = [ (a, None, None) for a in unicode(resource[u'creator']) ]
        links = [
            (unicode(resource['handle']), u'alternate'),
        ]
        #categories = [ (unicode(k), SD_NS+u'authorKeyword') for k in authkw(article) ]
        #elements = [
        #    E((SD_NS, u'sd:journal-cover'), unicode(article.journalCover).strip() if hasattr(article, 'journalCover') else DEFAULT_ICON),
        #    E((SD_NS, u'sd:journal-name'), unicode(article.journalName)),
        #]
        f.append(
            id,
            unicode(resource['title'][0]),
            updated=unicode(resource['date'][0]),
            summary=unicode(resource['description'][0]),
            authors=authors,
            links=links,
            #categories=categories,
            #elements=elements,
        )

    return f.source.xml_encode('xml-indent')




# Test akara.dist.setup()
import sys
import os
import tempfile
import subprocess
import shutil

from akara import dist

class SetupException(Exception):
    pass

# Do a bit of extra work since nosetests might run in the top-level
# Akara directory or in test/ .
dirname = os.path.dirname(__file__)
setup_scripts_dir = os.path.join(dirname, "setup_scripts")
assert os.path.isdir(setup_scripts_dir), setup_scripts_dir

def call_setup(args):
    p = subprocess.Popen([sys.executable] + args,
                         stdout = subprocess.PIPE,
                         stderr = subprocess.STDOUT,
                         cwd=setup_scripts_dir)
    stdout = p.stdout.read()
    p.wait()
    if p.returncode != 0:
        raise SetupException("setup.py failure %d: %s" % (p.returncode, stdout,))
    # print here to help in case of failures;
    # nosetests prints captured stdout
    print stdout

def test_basic():
    dirname = tempfile.mkdtemp(prefix="akara_setup_test_")
    try:
        call_setup(["setup_basic.py", "install",
                    "--root", dirname,
                    "--akara-modules-dir", dirname])
        assert os.path.exists(os.path.join(dirname, "blah.py"))
    finally:
        shutil.rmtree(dirname)
    
def test_missing():
    try:
        call_setup(["setup_missing.py", "install",
                    "--root", "/../this/does/not/exist",
                    "--akara-modules-dir", "/../this/does/not/exist"])
        raise AssertionError
    except SetupException, err:
        s = str(err)
        assert "you need to include the 'akara_extensions' parameter" in s, s


def test_bad_ext():
    try:
        call_setup(["setup_bad_ext.py", "install",
                    "--root", "/../this/does/not/exist",
                    "--akara-modules-dir", "/../this/does/not/exist"])
        raise AssertionError
    except SetupException, err:
        s = str(err)
        assert "Akara extensions must end with '.py'" in s, s

def test_specifying_config():
    dirname = tempfile.mkdtemp(prefix="akara_setup_test_")
    config_filename = os.path.join(dirname, "akara.conf")
    try:
        f = open(config_filename, "w")
        f.write("class Akara: ConfigRoot = %r + '/blather'\n" % dirname)
        f.close()
            
        call_setup(["setup_basic.py", "install",
                    "--root", dirname,
                    "--akara-config", config_filename])
        assert os.path.exists(os.path.join(dirname, "blather", "modules", "blah.py"))
    finally:
        shutil.rmtree(dirname)

# dirname has priority
def test_specifying_config_and_dir():
    dirname = tempfile.mkdtemp(prefix="akara_setup_test_")
    try:
        try:
            call_setup(["setup_basic.py", "install",
                        "--root", dirname,
                        "--akara-config", "setup_akara.conf",
                        "--akara-modules-dir", dirname])
            assert os.path.exists(os.path.join(dirname, "blah.py"))
        except SetupException, err:
            s = str(err)
            assert "flapdoodle" in s, s
    finally:
        shutil.rmtree(dirname)



# Test internal Akara code

from akara.services import convert_body
from amara import tree

# Found a problem in the convert_body code. Returned the XML as a
# string instead of a list containing a single string. Ruined the
# throughput as WSGI processed the result character-by-character.


test_tree = tree.entity()
test_tree.xml_append(tree.element(None, 'spam'))

def test_convert_body_string():
    result = convert_body("Hello", None, None, None)
    assert result == (["Hello"], "text/plain", 5), result

    result = convert_body("Hello", "text/not-plain", None, None)
    assert result == (["Hello"], "text/not-plain", 5), result

def test_convert_body_unicode():
    result = convert_body(u"G\u00f6teborg", None, "utf8", None)
    assert result == (["G\xc3\xb6teborg"], "text/plain; charset=utf8", 9), result

    result = convert_body(u"G\u00f6teborg", "text/not-plain", "utf16", None)
    assert result == (["\xff\xfeG\x00\xf6\x00t\x00e\x00b\x00o\x00r\x00g\x00"], "text/not-plain", 18), result


def test_convert_body_xml():
    result = convert_body(test_tree, None, "utf8", "xml")
    assert result == (['<?xml version="1.0" encoding="utf8"?>\n<spam/>'], "application/xml", 45), result

def test_convert_body_html():
    result = convert_body(test_tree, None, "utf8", "html")
    assert result == (["<spam></spam>"], "text/html", 13), result

def test_convert_body_list():
    result = convert_body(["blah"], None, None, None)
    assert result == (["blah"], "text/plain", None), result




# -*- encoding: utf-8 -*-
'''
See also:
'''

import re, os, time
import calendar
from datetime import date; from dateutil.relativedelta import *
from itertools import *
from wsgiref.util import shift_path_info, request_uri

from dateutil.parser import parse

import amara
from amara.lib.iri import join

from akara.services import simple_service
from akara import request
from akara import logger

from string import Template

CAL_TEMPLATE = Template('''
<table class="akaraCalCalendar" xmlns="http://www.w3.org/1999/xhtml">
  <thead>
    <tr class="akaraCalCalendarTopHeaders">
      $prevmonth<th colspan="5">$monthname, $year</th>$nextmonth
    </tr>
    <tr class="akaraCalCalendarWeekHeaders">
      $dayheaders
    </tr>
  </thead>
  <tbody>
    $month
  </tbody>
</table>
''')

SERVICE_ID = 'http://purl.org/xml3k/akara/services/demo/calendar'
@simple_service('GET', SERVICE_ID, 'akara.calendar', 'text/html') #application/xhtml+xml
def akara_calendar(highlight=None):
    '''
    Return a calendar in HTML
    Generates a calendar along the lines of:

        <  January, 2007   >
        Mo Tu We Th Fr Sa Su
               1  2  3  4  5
         6  7  8  9 10 11 12
        13 14 15 16 17 18 19
        20 21 22 23 24 25 26
        27 28 29 30 31

    Marks present date and those that have entries with archive links

    Defines the following classes (for use in CSS customization):

      - akaraCalCalendar
        - calendar table (note: month/year header e.g. January 2007 is in table/th)
      - akaraCalCalendarWeekHeaders
        - week header (Su, Mo, Tu, ...)
      - akaraCalCalendarEmpty
        - filler cell (e.g. days after Jan 31)
      - akaraCalCalendarLive
        - day for which there is an entry (also has links to that day's archives)

    And the following IDs:

      - akaraCalCalendarToday
        - today's calendar day
      - akaraCalCalendarSpecificDay
        - specific day being rendered (if any)

    Some ideas (e.g. CSS styling of the table) from pycalendar.py by Will Guaraldi

    Sample request:
    curl http://localhost:8880/akara.calendar
    curl http://localhost:8880/akara.calendar/2008/12
    curl http://localhost:8880/akara.calendar/2008/12?highlight=2008-12-03
    '''
    baseuri = request.environ['SCRIPT_NAME'] + '/'
    today = date.today()
    year = shift_path_info(request.environ)
    month = shift_path_info(request.environ)
    if highlight:
        #Fun axiom: date(*map(int, date.today().isoformat().split('-')))
        highlight = date(*map(int, highlight.split('-')))
    if year and month:
        #Use specified year & month
        year, month = int(year), int(month)
        if (year, month) == (today.year, today.month):
            present_day = today.day
        else:
            present_day = None
    else:
        #XXX We might want to return Bad Request of they specified year but not day
        #Use present year & month
        year, month = today.year, today.month
        present_day = today.day
    #logger.debug("year: " + repr(year))

    dayheaders = ''.join(
        ['<td>%s</td>' % dh
         for dh in calendar.weekheader(3).split()]
    )
    monthcal = calendar.monthcalendar(year, month)
    c = []
    for wk in monthcal:
        c.append('<tr>\n')
        for d in wk:
            d_int = int(d)
            attrs = ''
            if d_int < 1:
                d = '&#160;'
                fulldate = date.max #never to be found in archives
                attrs += ' class="akaraCalCalendarEmpty"'
            else:
                fulldate = date(year, month, d_int)
            # "today" trumps "specific day"
            if d_int == present_day:
                attrs += ' id="akaraCalCalendarToday"'
            elif highlight and d_int == highlight.day:
                attrs += ' id="akaraCalCalendarSpecificDay"'
            #if fulldate in archives:
            #    attrs += ' class="akaraCalCalendarLive"'
                #d = '<a href="%s%i/%i/%s/">%s</a>'%(self.weblog_base_url, year, month, d, d)
            #    d = '%s'%(d)
            c.append('\t<td%s>%s</td>\n' % (attrs, d))
        c.append('\n</tr>\n')
    monthname =  calendar.month_name[month]
    prevmonth = date(year, month, 1) + relativedelta(months=-1)
    nextmonth = date(year, month, 1) + relativedelta(months=+1)
    #Yes, even checking if prevmonth > today, so if someone surfs
    #3 month in the future, there will be no month nav links
    if prevmonth > today:
        prevmonth = ''
    else:
        #prevmonth = '<th><a href="%s%i/%i/">&lt;&lt;</a></th>'%(self.weblog_base_url, prevmonth.year, prevmonth.month)
        prevmonth = '<th><a href="%s">&lt;&lt;</a></th>'%(join(baseuri, str(prevmonth.year), str(prevmonth.month)))
    if nextmonth > today:
        nextmonth = ''
    else:
        nextmonth = '<th><a href="%s">&gt;&gt;</a></th>'%(join(baseuri, str(nextmonth.year), str(nextmonth.month)))
    month = ''.join(c)
    cal = CAL_TEMPLATE.safe_substitute(locals())
    return cal

